/****************************************************************************
 *                                                                          *
 *                  Concours 2006 - GAMES CREATORS NETWORK                  *
 *                                                                          *
 ****************************************************************************

                     PRODUCT          : GRAVITY GAME
 
                     FILE             : Animation.cpp

                     VERSION          : 1
 
 ****************************************************************************
 *                                                                          *
 *  DESCRIPTION :                                                           *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          * 
 *                                                                          *  
 ****************************************************************************
 *                                                                          *
 * HISTORY :                                                                * 
 * ---------                                                                *
 *                                                                          *
 *  Date        | Author  | Comment                                         *
 *  ---------------------------------------------------------------------   *
 *  28/07/2006  | Bahamut | Creation                                        *
 *  ---------------------------------------------------------------------   *
 *                                                                          *
 *                                                                          *
 ****************************************************************************/


#include <stdlib.h>
#include <math.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <GL/glew.h>


#include "Animation.h"



/****************************************************************************
 *                                                                          *
 *    Function     : CAnimation                                             * 
 *                                                                          *
 *    Description  : Class construtor                                       *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
CAnimation::CAnimation(float x, float y, float z, int width, int height, std::string texName, int number)
{
	m_vPosition       = CPoint3D<float>(x, y, z);
	m_fWidth          = width;
	m_fHeight         = height;
	m_iPicturesNumber = number;

	m_iCurrentIndex   = 0;
	m_iCurrentTime    = 0;
	m_iTimeStep       = 100; // 100ms == 10 Frames per second
	                         //  50ms == 20 Frames per second

	CTextureManager *pTex = CTextureManager::GetInstance();
	for(int i=0; i<m_iPicturesNumber; i++)
	{
		char textureName[50];
		sprintf(textureName,"%s%d",texName.c_str(), i);
		//std::string textureName = texName + ss.str();
		//textureName.append(i);
		
		m_iTextureID[i] = pTex->index(textureName);
	}
}


/****************************************************************************
 *                                                                          *
 *    Function     : ~CAnimation                                            * 
 *                                                                          *
 *    Description  : Class destructor                                       *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
CAnimation::~CAnimation(void)
{
}


/****************************************************************************
 *                                                                          *
 *    Function     : Update                                                 * 
 *                                                                          *
 *    Description  : Updates the animation                                  *
 *                                                                          *
 *    Interface    : I time = time elapsed                                  *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CAnimation::Update(Uint32 time)
{
	m_iCurrentTime += time;
	if ( m_iCurrentTime >= m_iTimeStep )
	{
		m_iCurrentTime -= m_iTimeStep;
		m_iCurrentIndex++;
		if (m_iCurrentIndex >= m_iPicturesNumber)
			m_iCurrentIndex = 0;
	}
}


/****************************************************************************
 *                                                                          *
 *    Function     : Draw                                                   *
 *                                                                          *
 *    Description  : Draws the animation                                    *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CAnimation::Draw(void)
{
		glBindTexture(GL_TEXTURE_2D, m_iTextureID[m_iCurrentIndex]);

		glBegin( GL_QUADS );
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f, 0.0f);
				glVertex3f(m_vPosition.X(), m_vPosition.Y()+m_fHeight, m_vPosition.Z());
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f, 1.0f);
				glVertex3f(m_vPosition.X(), m_vPosition.Y(), m_vPosition.Z());
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f, 1.0f);
				glVertex3f(m_vPosition.X()+m_fWidth, m_vPosition.Y(), m_vPosition.Z());
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f, 0.0f);
				glVertex3f(m_vPosition.X()+m_fWidth, m_vPosition.Y()+m_fHeight, m_vPosition.Z());
		glEnd();
}



void CAnimation::SetWidth(float width)
{
	m_fWidth = width;
}

void CAnimation::SetHeight(float height)
{
	m_fHeight = height;
}

void CAnimation::SetTimeStep(int timestep)
{
	m_iTimeStep = timestep;
}

void CAnimation::SetPosition(CPoint3D<float> position)
{
	m_vPosition = position;
}


float CAnimation::Width()
{
	return m_fWidth;
}

float CAnimation::Height()
{
	return m_fHeight;
}

CPoint3D<float> CAnimation::Position()
{
	return m_vPosition;
}

int CAnimation::TimeStep()
{
	return m_iTimeStep;
}

int CAnimation::PicturesNumber()
{
	return m_iPicturesNumber;
}







/****************************************************************************
 *                                                                          *
 *  NOTES :                                                                 *
 *                                                                          *
 ****************************************************************************
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *    
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *         
 *                                                                          *    
 ****************************************************************************/ 
 