/****************************************************************************
 *                                                                          *
 *                  Concours 2006 - GAMES CREATORS NETWORK                  *
 *                                                                          *
 ****************************************************************************

                     PRODUCT          : GRAVITY GAME
 
                     FILE             : Flotte.cpp

                     VERSION          : 1
 
 ****************************************************************************
 *                                                                          *
 *  DESCRIPTION :                                                           *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          * 
 *                                                                          *  
 ****************************************************************************
 *                                                                          *
 * HISTORY :                                                                * 
 * ---------                                                                *
 *                                                                          *
 *  Date        | Author  | Comment                                         *
 *  ---------------------------------------------------------------------   *
 *  24/07/2006  | Bahamut | Creation                                        *
 *  ---------------------------------------------------------------------   *
 *                                                                          *
 *                                                                          *
 ****************************************************************************/


#include <stdlib.h>
#include <math.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <GL/glew.h>

#pragma warning( disable : 4305 ) //truncation from 'double' to 'GLfloat'
#pragma warning( disable : 4244 ) //conversion from 'double' to 'float'


/////////////////////////////////////////////////
// fluid object definition                     //
/////////////////////////////////////////////////

#include "flotte.h"


/****************************************************************************
 *                                                                          *
 *    Function     : dmFlotte                                               * 
 *                                                                          *
 *    Description  : Class construtor                                       *
 *                                                                          *
 *    Interface    : I firstTexture  = texture identifiant number 1         *
 *                   I secondTexture = texture identifiant number 2         *
 *                   I rez           = level of detail                      *  
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
dmFlotte::dmFlotte(int firstTexture, int secondTexture, int rez)
{
   // initialize some fields
   _angle = 0;

   // initial conditions
   _p1 = &_Flot1; // FRONT MAP
   _p2 = &_Flot2; // BACK MAP

	texture1 = firstTexture;
	texture2 = secondTexture;

	dmFlotte::init_Flot();

	// by default high tesselation

   _lowtesselation = false;
   _hightesselation = true;

   if (rez == FLOT_MEDIUMRES) 
     _hightesselation = false;
   else if (rez == FLOT_LOWRES)
     _lowtesselation = true; 

   // prebuild geometric model
   dmFlotte::_prebuild_Flot();
   dmFlotte::_init_GfxContext(); // initialize gfx context

   // current clock
   _last_msTime = ( (clock() * 1000) / CLOCKS_PER_SEC ) - 100.0f;
   _cur_msTime = _last_msTime + 100.0f;
	#if __DEVELOPMENT__
		TRACE0(__TERRA__,MSG_0017);
	#endif   
}


/****************************************************************************
 *                                                                          *
 *    Function     : ~dmFlotte                                              * 
 *                                                                          *
 *    Description  : Class Destrutor                                        *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
dmFlotte::~dmFlotte(void)
{
}


/****************************************************************************
 *                                                                          *
 *    Function     : _init_GfxContext                                       * 
 *                                                                          *
 *    Description  : Initializes graphics context                           *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void dmFlotte::_init_GfxContext(void)
{
    // * Zbuff on ... *
    glEnable(GL_DEPTH_TEST);
    glDepthFunc(GL_LESS);

    // no face culling to have recto-verso
    glShadeModel(GL_SMOOTH); // shading Gouraud

    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);

    // scale projection matrix for vertices... 
    glMatrixMode(GL_MODELVIEW);
    glScalef(.01, .01, .01);
}


/****************************************************************************
 *                                                                          *
 *    Function     : init_Flot                                              * 
 *                                                                          *
 *    Description  : Reset every heights at zero                            *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void dmFlotte::init_Flot(void)
{
    memset((unsigned char*)(*_p1), 0, sizeof(int)*(FLOTSIZE+2)*(FLOTSIZE+2));
	memset((unsigned char*)(*_p2), 0, sizeof(int)*(FLOTSIZE+2)*(FLOTSIZE+2));
}


/****************************************************************************
 *                                                                          *
 *    Function     : setWave                                                * 
 *                                                                          *
 *    Description  : Traces a hole at normalized coordinates                *
 *                                                                          *
 *    Interface    : I aX   = position of the wave at x                     *
 *                   I aY   = position of the wave at y                     *
 *                   I aVal = value of the wave                             *
 *                                                                          *  
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void dmFlotte::setWave(float aX, float aY, int aVal)
{
    int x = FLOT_MILX + (int)(2.0f*(aX - 0.5f)*FLOT_LARG);
    int y = FLOT_MILY + (int)(2.0f*(aY - 0.5f)*FLOT_LARG);

	// check periodicity
    while (x > FLOTSIZE)  x -= FLOTSIZE;
    while (y > FLOTSIZE)  y -= FLOTSIZE;
    while (x < 0) x += FLOTSIZE;
    while (y < 0) y += FLOTSIZE;

    (*_p1)[x][y] -= aVal;
}

//-----------------------------------
// trace a hole following parametric
// curves
//-----------------------------------

void dmFlotte::runWave(float aPhase, float aCos, float aSin, int aVal)
{
    float r = (_angle*M_PI)/1024.0f;

    int x = FLOT_MILX + ((int) (cos(aCos*r + aPhase)*FLOT_LARG));
    int y = FLOT_MILY + ((int) (sin(aSin*r + aPhase)*FLOT_LARG));

    if (x > FLOTSIZE)  x = FLOTSIZE;
    if (y > FLOTSIZE)  y = FLOTSIZE;
    if (x < 0) x = 0;
    if (y < 0) y = 0;

    (*_p1)[x][y] -= aVal;
}


/****************************************************************************
 *                                                                          *
 *    Function     : randomWave                                             * 
 *                                                                          *
 *    Description  : Makes a Random hole                                    *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void dmFlotte::randomWave(void)
{
    (*_p1)[rand()%FLOTSIZE+1][rand()%FLOTSIZE+1] -= rand()%128;
}


/****************************************************************************
 *                                                                          *
 *    Function     : bench                                                  * 
 *                                                                          *
 *    Description  : Measures elapsed time                                  *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
bool dmFlotte::bench(float rate)
{
	_cur_msTime = (clock() * 1000) / CLOCKS_PER_SEC;

    // don't run to quick otherwise it doesn't look like water
	return (_cur_msTime - _last_msTime) >= rate;
}


/****************************************************************************
 *                                                                          *
 *    Function     : nexttime                                               * 
 *                                                                          *
 *    Description  :                                                        *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void dmFlotte::nexttime(void)
{
	_last_msTime = _cur_msTime;
}


/****************************************************************************
 *                                                                          *
 *    Function     : update                                                 * 
 *                                                                          *
 *    Description  : Updates to next state of fluid model                   *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void dmFlotte::update(void)
{

    // new angle for parametric curves
    _angle = (_angle+2) & 1023;


    // fluid update
    dmFlotte::_new_Flot();

    // smoothing
    dmFlotte::_Lisse_Flot();
}


/****************************************************************************
 *                                                                          *
 *    Function     : build                                                  * 
 *                                                                          *
 *    Description  : Builds geometric model                                 *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void dmFlotte::build(void)
{
    dmFlotte::_build_Flot();
}


/****************************************************************************
 *                                                                          *
 *    Function     : _new_Flot                                              * 
 *                                                                          *
 *    Description  : Physical calculus for fluid model                      *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void dmFlotte::_new_Flot(void)
{
     register int x,y, step;
     register int *ptr;
     int (*q)[FLOTSIZE+2][FLOTSIZE+2];

     // discretized differential equation
     for(x=1; x<=FLOTSIZE; x++)
     {
        for(y=0; y<=FLOTSIZE; y ++)
        {
		   (*_p1)[x][y] = (((*_p2)[x-1][y]+(*_p2)[x+1][y]+(*_p2)[x][y-1]+(*_p2)[x][y+1]) >> 1) - (*_p1)[x][y];
           (*_p1)[x][y] -= (*_p1)[x][y] >> 4;
        }
     }

     // copy borders to make the map periodic
     memcpy(&((*_p1)[0][0]), &((*_p1)[1][0]), sizeof(int)*(FLOTSIZE+2));
     memcpy(&((*_p1)[FLOTSIZE+1][0]), &((*_p1)[1][0]), sizeof(int)*(FLOTSIZE+2));

     step = (FLOTSIZE+2);

     for(x=0, ptr = &((*_p1)[0][0]); x<(FLOTSIZE+1); x++, ptr += step)
     {
  		ptr[0] = ptr[1];
		ptr[FLOTSIZE+1] = ptr[1];
     }

     /* swap buffers t and t-1, we advance in time */
     q=_p1;
     _p1=_p2;
     _p2=q;
}


/****************************************************************************
 *                                                                          *
 *    Function     : _Lisse_Flot                                            * 
 *                                                                          *
 *    Description  : Filter and smooth producted values                     *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void dmFlotte::_Lisse_Flot(void)
{
     register int x,y,i;


     for(x=1; x<FLOTSIZE+1; x++)
     {
        for(y=1; y<FLOTSIZE+1; y++)
        {
           _Lisse[x][y] = (3*(*_p1)[x][y]+2*(*_p1)[x+1][y]+2*(*_p1)[x][y+1]+(*_p1)[x+1][y+1]) >> 3;
		}
     }

     for(i=1; i<4; i++)
     {
       for(x=1; x<FLOTSIZE+1; x++)
       {
          for(y=1; y<FLOTSIZE+1; y++)
          {
             _Lisse[x][y] = (3*_Lisse[x][y]+2*_Lisse[x+1][y]+2*_Lisse[x][y+1]+_Lisse[x+1][y+1]) >> 3;
          }
       }
     }
}


/****************************************************************************
 *                                                                          *
 *    Function     : _prebuild_Flot                                         * 
 *                                                                          *
 *    Description  : Pre-building of a geometric model                      *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void dmFlotte::_prebuild_Flot(void)
{

     register float xmin,ymin;
     register int x,y;

	 if (_hightesselation) 
     {
        // vertices calculus -> we already know X and Y
	    // calculus of background texture coordinates 
        for(x=1; x<=FLOTSIZE; x++)
		{
           xmin = (x-FLOT_MILX)/6.5f;

           for(y=1; y<=FLOTSIZE; y++)
		   {
			  ymin = (y-FLOT_MILY)/7.5f;

              _sommet[(x-1) *2][(y-1) *2][0] = xmin;
              _sommet[(x-1) *2][(y-1) *2][1] = ymin;

		      _uvmap[(x-1) *2][(y-1) *2][0] = (x-1)*(1.0f/(FLOTSIZE-1));
              _uvmap[(x-1) *2][(y-1) *2][1] = (y-1)*(1.0f/(FLOTSIZE-1));
		   }

		}
     

        // build vertices in-between
        for(x=0;  x<= FLOTSIZE*2 - 1; x+=2)     // even rows
		{
           for(y=1; y<=FLOTSIZE*2 - 2;  y+=2)   // odd columns
		   {
              _sommet[x][y][0]  = (_sommet[x][y-1][0] + _sommet[x][y+1][0]) / 2.0f;
              _sommet[x][y][1]  = (_sommet[x][y-1][1] + _sommet[x][y+1][1]) / 2.0f;
              _uvmap[x][y][0]  = (_uvmap[x][y-1][0] + _uvmap[x][y+1][0]) / 2.0f;
              _uvmap[x][y][1]  = (_uvmap[x][y-1][1] + _uvmap[x][y+1][1]) / 2.0f;
		   }

		}

        // build vertices in-between
        for(x=1;  x<= FLOTSIZE*2 - 2; x+=2)    // odd rows
		{
           for(y=0; y<=FLOTSIZE*2 - 1;  y++)   // every columns
		   {
              _sommet[x][y][0]  = (_sommet[x-1][y][0] + _sommet[x+1][y][0]) / 2.0f;
              _sommet[x][y][1]  = (_sommet[x-1][y][1] + _sommet[x+1][y][1]) / 2.0f;
              _uvmap[x][y][0]  = (_uvmap[x-1][y][0] + _uvmap[x+1][y][0]) / 2.0f;
              _uvmap[x][y][1]  = (_uvmap[x-1][y][1] + _uvmap[x+1][y][1]) / 2.0f;
		   }

		}

        // nomals to faces calculus : Z component is constant

        // -> simplified cross product and optimized knowing that we have
        //    a distance of 1.0 between each fluid cells.

        for(x=0; x < (FLOTSIZE << 1)-1; x++)
		{
           for(y=0; y < (FLOTSIZE << 1)-1; y++)
		   {
              _normal[x][y][2] = 0.01f;
		   }
		}

        //..............................................................................................................
        //
        // the following calculus is useless because each cell of _snormal[x][y][2] = 0.01+0.01+0.01+0.01 = 0.04
        //..............................................................................................................


        // copy borders of the map (Z component only) for periodicity 
        memcpy((char*) &_normal[FLOTSIZE*2-1][0][0], (char*) &_normal[FLOTSIZE*2-2][0][0], 3*sizeof(float)*FLOTSIZE*2);

        for(x=0; x < FLOTSIZE*2; x++)
		{
   	       _normal[x][FLOTSIZE*2-1][2] = _normal[x][FLOTSIZE*2-2][2];
		}


        // calculate normals to vertices (Z component only)
        for(x=1; x < FLOTSIZE*2-1; x++)
		{
           for(y=1; y < FLOTSIZE*2-1; y++)
		   {
              _snormal[x][y][2] = _normal[x-1][y][2]+_normal[x+1][y][2]+_normal[x][y-1][2]+_normal[x][y+1][2];
		   }
		}

        // copy borders of the map (Z component only) for periodicity 
        for(x=0; x < FLOTSIZE*2; x++)
		{
           _snormal[x][0][2] = _normal[x][0][2];
           _snormal[x][FLOTSIZE*2-1][2] = _normal[x][FLOTSIZE*2-1][2];
		}

        memcpy((char*) &_snormal[0][0][0], (char*) &_normal[0][0][0], 3*sizeof(float)*FLOTSIZE*2);
        memcpy((char*) &_snormal[FLOTSIZE*2-1][0][0], (char*) &_normal[FLOTSIZE*2-1][0][0], 3*sizeof(float)*FLOTSIZE*2);

   } // end hightesselation
   else
   {
        // vertices calculus -> we already know X and Y 
	    // calculus of background texture coordinates 

        for(x=1; x<=FLOTSIZE; x++)
		{
           xmin = (x-FLOT_MILX)/6.5f;

           for(y=1; y<=FLOTSIZE; y++)
		   {
			  ymin = (y-FLOT_MILY)/7.5f;

              _sommet[x-1][y-1][0] = xmin;
              _sommet[x-1][y-1][1] = ymin;

		      _uvmap[x-1][y-1][0] = (x-1)*(1.0f/(FLOTSIZE-1));
              _uvmap[x-1][y-1][1] = (y-1)*(1.0f/(FLOTSIZE-1));
		   }

		}
     
		// nomals to faces calculus : Z component is constant

        // -> simplified cross product and optimized knowing that we have
        //    a distance of 1.0 between each fluid cells.
        for(x=0; x < FLOTSIZE-1; x++)
		{
           for(y=0; y < FLOTSIZE-1; y++)
		   {
              _normal[x][y][2] = 0.01f;
		   }
		}

		//..............................................................................................................
        //
        // the following calculus is useless because each cell of _snormal[x][y][2] = 0.01+0.01+0.01+0.01 = 0.04
        //..............................................................................................................


        // copy borders of the map (Z component only) for periodicity 
        memcpy((char*) &_normal[FLOTSIZE-1][0][0], (char*) &_normal[FLOTSIZE-2][0][0], 3*sizeof(float)*FLOTSIZE);

        for(x=0; x < FLOTSIZE; x++)
		{
   	       _normal[x][FLOTSIZE-1][2] = _normal[x][FLOTSIZE-2][2];
		}

        // calculate normals to vertices (Z component only)

        for(x=1; x < FLOTSIZE-1; x++)
		{
           for(y=1; y < FLOTSIZE-1; y++)
		   {
              _snormal[x][y][2] = _normal[x-1][y][2]+_normal[x+1][y][2]+_normal[x][y-1][2]+_normal[x][y+1][2];
		   }
		}

        // copy borders of the map (Z component only) for periodicity 
        for(x=0; x < FLOTSIZE; x++)
		{
           _snormal[x][0][2] = _normal[x][0][2];
           _snormal[x][FLOTSIZE-1][2] = _normal[x][FLOTSIZE-1][2];
		}

        memcpy((char*) &_snormal[0][0][0], (char*) &_normal[0][0][0], 3*sizeof(float)*FLOTSIZE);
        memcpy((char*) &_snormal[FLOTSIZE-1][0][0], (char*) &_normal[FLOTSIZE-1][0][0], 3*sizeof(float)*FLOTSIZE);

   } // end lowtesselation
 
}


/****************************************************************************
 *                                                                          *
 *    Function     : _build_Flot                                            * 
 *                                                                          *
 *    Description  : Construction of a geometric model                      *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void dmFlotte::_build_Flot(void)
{

     register float h1,sqroot;
     register int x,y;

	 if (_hightesselation)
     {

        // calculate vertices : Z component 

        for(x=1; x<=FLOTSIZE; x++)
		{
           for(y=1; y<=FLOTSIZE; y++)
		   {
              if ( (h1 = (_Lisse[x][y] / 100.0f)) < 0.0f)
			  {
	             h1 = 0.0f;
			  }

              _sommet[(x-1) << 1][(y-1) << 1][2] = h1;
		   }
		}

        // construct vertices in-between
        for(x=0;  x<= FLOTSIZE*2 - 1; x+=2)     // even rows
		{
           for(y=1; y<=FLOTSIZE*2 - 2;  y+=2)   // odd columns
		   {
              _sommet[x][y][2]  = (_sommet[x][y-1][2] + _sommet[x][y+1][2]) / 2.0f;
		   }
		}

        // construct vertices in-between
        for(x=1;  x<= FLOTSIZE*2 - 2; x+=2)    // even rows
		{
           for(y=0; y<=FLOTSIZE*2 - 1;  y++)   // every columns
		   {
              _sommet[x][y][2]  = (_sommet[x-1][y][2] + _sommet[x+1][y][2]) / 2.0f;
		   }
		}

        //calculate normals to faces : components X and Y 

        // -> simplified cross product knowing that we have a distance of 1.0 between
        //    each fluid cells.
        for(x=0; x < FLOTSIZE*2 -1; x++)
		{
           for(y=0; y < FLOTSIZE*2 -1; y++)
		   {
              _normal[x][y][0] = 0.1f*(_sommet[x][y][2]-_sommet[x+1][y][2]);
              _normal[x][y][1] = 0.1f*(_sommet[x][y][2]-_sommet[x][y+1][2]);
		   }
		}

        // copy map borders(components X and Y only) for periodicity
        memcpy((char*) &_normal[FLOTSIZE*2-1][0][0], (char*) &_normal[FLOTSIZE*2-2][0][0], 3*sizeof(float)*FLOTSIZE*2);

        for(x=0; x < FLOTSIZE*2; x++)
		{
           _normal[x][FLOTSIZE*2-1][0] = _normal[x][FLOTSIZE*2-2][0];
	       _normal[x][FLOTSIZE*2-1][1] = _normal[x][FLOTSIZE*2-2][1];
		}

        // calculate normals to vertices (components X and Y only) 
        for(x=1; x < FLOTSIZE*2-1; x++)
		{
           for(y=1; y < FLOTSIZE*2-1; y++)
		   {
              _snormal[x][y][0] = _normal[x-1][y][0]+_normal[x+1][y][0]+_normal[x][y-1][0]+_normal[x][y+1][0];
              _snormal[x][y][1] = _normal[x-1][y][1]+_normal[x+1][y][1]+_normal[x][y-1][1]+_normal[x][y+1][1];
		   }
		}

        // copy map borders (components X and Y only)
        for(x=0; x < FLOTSIZE*2; x++)
		{
           _snormal[x][0][0] = _normal[x][0][0];
           _snormal[x][0][1] = _normal[x][0][1];
           _snormal[x][FLOTSIZE*2-1][0] = _normal[x][FLOTSIZE*2-1][0];
           _snormal[x][FLOTSIZE*2-1][1] = _normal[x][FLOTSIZE*2-1][1];
		}

        memcpy((char*) &_snormal[0][0][0], (char*) &_normal[0][0][0], 3*sizeof(float)*FLOTSIZE*2);
        memcpy((char*) &_snormal[FLOTSIZE*2-1][0][0], (char*) &_normal[FLOTSIZE*2-1][0][0], 3*sizeof(float)*FLOTSIZE*2);

        // calculate ourself normalization
        for(x=0; x < FLOTSIZE*2; x++)
		{
           for(y=0; y < FLOTSIZE*2; y++)
		   {
              sqroot = sqrt(_snormal[x][y][0]*_snormal[x][y][0] +
			                _snormal[x][y][1]*_snormal[x][y][1] +
						    0.0016f);

	          _snormaln[x][y][0] = _snormal[x][y][0]/sqroot;
	          _snormaln[x][y][1] = _snormal[x][y][1]/sqroot;
	          _snormaln[x][y][2] = 0.04f/sqroot;             // _snormal[x][y][2] = 0.04


		      // perturbate coordinates of background mapping with the components X,Y of normals...
              // simulate refraction

		      _newuvmap[x][y][0] = _uvmap[x][y][0] + 0.05f*_snormaln[x][y][0];
		      _newuvmap[x][y][1] = _uvmap[x][y][1] + 0.05f*_snormaln[x][y][1];

		   }
		}

        // really simple version of a fake envmap generator

        for(x=0; x < FLOTSIZE*2; x++)
		{
           for(y=0; y < FLOTSIZE*2; y++)
		   {
              // trick : xy projection of normals  ->  assume reflection in direction of the normals
		      //                                       looks ok for non-plane surfaces
		      _envmap[x][y][0] = 0.5f + _snormaln[x][y][0]*0.45f;
		      _envmap[x][y][1] = 0.5f + _snormaln[x][y][1]*0.45f;

		   }
		}

   } // end high tesselation
   else
   {
        // calculate vertices : Z component

        for(x=1; x<=FLOTSIZE; x++)
		{
           for(y=1; y<=FLOTSIZE; y++)
		   {
              if ( (h1 = (_Lisse[x][y] / 100.0f)) < 0.0f)
			  {
	             h1 = 0.0f;
			  }

              _sommet[x-1][y-1][2] = h1;
		   }
		}


        // calculate normals to faces : components X and Y

        // -> simplified cross product knowing that we have a distance of 1.0 between
        //    each fluid cells.
        for(x=0; x < FLOTSIZE-1; x++)
		{
           for(y=0; y < FLOTSIZE-1; y++)
		   {
              _normal[x][y][0] = 0.1f*(_sommet[x][y][2]-_sommet[x+1][y][2]);
              _normal[x][y][1] = 0.1f*(_sommet[x][y][2]-_sommet[x][y+1][2]);
		   }
		}

        // copy map borders(components X and Y only) for periodicity 
        memcpy((char*) &_normal[FLOTSIZE-1][0][0], (char*) &_normal[FLOTSIZE-2][0][0], 3*sizeof(float)*FLOTSIZE);

        for(x=0; x < FLOTSIZE; x++)
		{
           _normal[x][FLOTSIZE-1][0] = _normal[x][FLOTSIZE-2][0];
	       _normal[x][FLOTSIZE-1][1] = _normal[x][FLOTSIZE-2][1];
		}

        // calculate normals to vertices (components X and Y only) 
        for(x=1; x < FLOTSIZE-1; x++)
		{
           for(y=1; y < FLOTSIZE-1; y++)
		   {
              _snormal[x][y][0] = _normal[x-1][y][0]+_normal[x+1][y][0]+_normal[x][y-1][0]+_normal[x][y+1][0];
              _snormal[x][y][1] = _normal[x-1][y][1]+_normal[x+1][y][1]+_normal[x][y-1][1]+_normal[x][y+1][1];
		   }
		}

        // copy map borders (components X and Y only)
        for(x=0; x < FLOTSIZE; x++)
		{
           _snormal[x][0][0] = _normal[x][0][0];
           _snormal[x][0][1] = _normal[x][0][1];
           _snormal[x][FLOTSIZE-1][0] = _normal[x][FLOTSIZE-1][0];
           _snormal[x][FLOTSIZE-1][1] = _normal[x][FLOTSIZE-1][1];
		}

        memcpy((char*) &_snormal[0][0][0], (char*) &_normal[0][0][0], 3*sizeof(float)*FLOTSIZE);
        memcpy((char*) &_snormal[FLOTSIZE-1][0][0], (char*) &_normal[FLOTSIZE-1][0][0], 3*sizeof(float)*FLOTSIZE);

        // calculate ourself normalization
        for(x=0; x < FLOTSIZE; x++)
		{
           for(y=0; y < FLOTSIZE; y++)
		   {
              sqroot = sqrt(_snormal[x][y][0]*_snormal[x][y][0] +
			                _snormal[x][y][1]*_snormal[x][y][1] +
						    0.0016f);

	          _snormaln[x][y][0] = _snormal[x][y][0]/sqroot;
	          _snormaln[x][y][1] = _snormal[x][y][1]/sqroot;
	          _snormaln[x][y][2] = 0.04f/sqroot;             // _snormal[x][y][2] = 0.04

		      // perturbate coordinates of background mapping with the components X,Y of normals...
              // simulate refraction
		      _newuvmap[x][y][0] = _uvmap[x][y][0] + 0.05f*_snormaln[x][y][0];
		      _newuvmap[x][y][1] = _uvmap[x][y][1] + 0.05f*_snormaln[x][y][1];

		   }
		}

        // really simple version of a fake envmap generator
        for(x=0; x < FLOTSIZE; x++)
		{
           for(y=0; y < FLOTSIZE; y++)
		   {
              // trick : xy projection of normals  ->  assume reflection in direction of the normals
		      //                                       looks ok for non-plane surfaces
		      _envmap[x][y][0] = 0.5f + _snormaln[x][y][0]*0.45f;
		      _envmap[x][y][1] = 0.5f + _snormaln[x][y][1]*0.45f;

		   }
		}
   } // end low tesselation
}


/****************************************************************************
 *                                                                          *
 *    Function     : display                                                * 
 *                                                                          *
 *    Description  : Displays geometric model                               *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void dmFlotte::display(void)
{
   int strip_width;

   if (_hightesselation)
	   strip_width = FLOTSIZE*2-1-1;	    
   else
	   strip_width = FLOTSIZE-1-1;

    // OpenGL specific
    register int x,y;

	glEnable(GL_TEXTURE_2D);
	// change blending
	glEnable(GL_BLEND);

	// use texture alpha-channel for blending
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

    glActiveTextureARB(GL_TEXTURE0_ARB);
    glBindTexture(GL_TEXTURE_2D, texture2); // 2nd texture -> background ..

    glActiveTextureARB(GL_TEXTURE1_ARB);
 	glBindTexture(GL_TEXTURE_2D, texture1); // 2nd texture -> envmap
    glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL); //GL_MODULATE // GL_REPLACE);

	// enable texture mapping and specify ourself texcoords
	glDisable(GL_TEXTURE_GEN_S);
    glDisable(GL_TEXTURE_GEN_T);

	// build triangle strips ...
	if (_lowtesselation)
    {
		for(x=0; x<strip_width; x+=2)
		{
     		glBegin(GL_TRIANGLE_STRIP);
			glMultiTexCoord2fvARB(GL_TEXTURE0_ARB, _newuvmap[x+2][2]);
			glMultiTexCoord2fvARB(GL_TEXTURE1_ARB, _envmap[x+2][2]);
			glVertex3fv(_sommet[x+2][2]);                               // otherwise everything is scrolled !!!

			for(y=2; y<strip_width; y+=2)
			{
 				glMultiTexCoord2fvARB(GL_TEXTURE0_ARB, _newuvmap[x][y]);
				glMultiTexCoord2fvARB(GL_TEXTURE1_ARB, _envmap[x][y]);
				glVertex3fv(_sommet[x][y]);

  				glMultiTexCoord2fvARB(GL_TEXTURE0_ARB, _newuvmap[x+2][y+2]);
				glMultiTexCoord2fvARB(GL_TEXTURE1_ARB, _envmap[x+2][y+2]);
				glVertex3fv(_sommet[x+2][y+2]);
			}

			glMultiTexCoord2fvARB(GL_TEXTURE0_ARB, _newuvmap[x][y]);
			glMultiTexCoord2fvARB(GL_TEXTURE1_ARB, _envmap[x][y]);
			glVertex3fv(_sommet[x][y]);

			glEnd();
		}

    }
	else // medium and high tesselation
    {
		for(x=0; x<strip_width; x++)
		{
     		glBegin(GL_TRIANGLE_STRIP);


			glMultiTexCoord2fvARB(GL_TEXTURE0_ARB, _newuvmap[x+1][1]);  // WARNING: glTexCoord2fv BEFORE glVertex !!!
			glMultiTexCoord2fvARB(GL_TEXTURE1_ARB, _envmap[x+1][1]);
			glVertex3fv(_sommet[x+1][1]);                               // otherwise everything is scrolled !!!

			for(y=1; y<strip_width; y++)
			{
 				glMultiTexCoord2fvARB(GL_TEXTURE0_ARB, _newuvmap[x][y]);
				glMultiTexCoord2fvARB(GL_TEXTURE1_ARB, _envmap[x][y]);
				glVertex3fv(_sommet[x][y]);

  				glMultiTexCoord2fvARB(GL_TEXTURE0_ARB, _newuvmap[x+1][y+1]);
				glMultiTexCoord2fvARB(GL_TEXTURE1_ARB, _envmap[x+1][y+1]);
				glVertex3fv(_sommet[x+1][y+1]);
			}


			glMultiTexCoord2fvARB(GL_TEXTURE0_ARB, _newuvmap[x][y]);
			glMultiTexCoord2fvARB(GL_TEXTURE1_ARB, _envmap[x][y]);
			glVertex3fv(_sommet[x][y]);

			glEnd();
		}
	}
}


/****************************************************************************
 *                                                                          *
 *  NOTES :                                                                 *
 *                                                                          *
 ****************************************************************************
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *    
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *         
 *                                                                          *    
 ****************************************************************************/ 
 