/****************************************************************************
 *                                                                          *
 *                  Concours 2006 - GAMES CREATORS NETWORK                  *
 *                                                                          *
 ****************************************************************************

                     PRODUCT          : GRAVITY GAME
 
                     FILE             : GameEngine.cpp

                     VERSION          : 1
 
 ****************************************************************************
 *                                                                          *
 *  DESCRIPTION :                                                           *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          * 
 *                                                                          *  
 ****************************************************************************
 *                                                                          *
 * HISTORY :                                                                * 
 * ---------                                                                *
 *                                                                          *
 *  Date        | Author  | Comment                                         *
 *  ---------------------------------------------------------------------   *
 *  24/07/2006  | Bahamut | Creation                                        *
 *  ---------------------------------------------------------------------   *
 *                                                                          *
 *                                                                          *
 ****************************************************************************/

#include "GameEngine.h"

int CGameEngine::s_iScreenWidth = 0;
int CGameEngine::s_iScreenHeight = 0;

/****************************************************************************
 *                                                                          *
 *    Function     : CGameEngine                                            * 
 *                                                                          *
 *    Description  : Constructeur                                           *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
CGameEngine::CGameEngine(void)
{
	// Valeurs par dfaut
	m_bRunning = false;
	m_bInited = false;
	m_bTexturesDebugMode = false;
	m_iTickCount = 0;
	m_iLastCount = 0;
	m_iVideoFlags = 0;
	CGameEngine::s_iScreenWidth = 1024;
	CGameEngine::s_iScreenHeight = 768;
	m_iScreenBpp = 0;
	m_oCurrentProcess = 0;
	m_bResizable = false;
	m_bFullScreen = true;
	
	m_oCurrentProcess = 0;




	// Initialisation de toutes les bibliothques (3D, son, physique, etc.)
	m_bInited = init();

	// OpenGL Extentions Test
	if (!checkOGLExtensions())
	{
		//TODO
	}

    // Cration de nos processus de jeu
	CBaseProcess* process;
	process = (CBaseProcess*) new CLoadingProcess();
	m_vProcess.push_back(process);

	process = (CBaseProcess*) new CIntroProcess();
	m_vProcess.push_back(process);

	process = (CBaseProcess*) new CMenuProcess();
	m_vProcess.push_back(process);

	process = (CBaseProcess*) new CGameProcess();
	m_vProcess.push_back(process);

	process = (CBaseProcess*) new CEndProcess();
	m_vProcess.push_back(process);
}


/****************************************************************************
 *                                                                          *
 *    Function     : ~CGameEngine                                           * 
 *                                                                          *
 *    Description  : Destructeur                                            *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
CGameEngine::~CGameEngine(void)
{
	// On dtruit tous nos processus de jeu
	std::vector<CBaseProcess*>::iterator it;
	while( !m_vProcess.empty() )
	{
		it = m_vProcess.begin();
		delete *it;
		m_vProcess.erase(it);
	}

	// On ferme toutes nos bibliothques (3D, son, physique, etc.)
	terminate();


	// On tue nos gestionnaires de son/texture
	CSoundManager::GetInstance()->Kill();
	CTextureManager::GetInstance()->Kill();

	// Termine la bibliothque OpenIl (Devil)
	ilShutDown();
}


/****************************************************************************
 *                                                                          *
 *    Function     : checkOGLExtensions                                     * 
 *                                                                          *
 *    Description  : Test le support de certaines extensions OpenGL         *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value : Retourne vrai si russi sinon retourne faux            *
 *                                                                          *
 ****************************************************************************/
bool CGameEngine::checkOGLExtensions(void)
{
	bool result = true;

	char *szGLExtensions = (char*)glGetString(GL_EXTENSIONS);
	//--------------------------- SHADERS ---------------------------//
	if(!strstr(szGLExtensions, "GL_ARB_shader_objects"))
		result = false;
	if(!strstr(szGLExtensions, "GL_ARB_shading_language_100"))
		result = false;
	//------------------------ MULTI TEXTURING ------------------------//
	if(!strstr(szGLExtensions, "GL_ARB_multitexture"))
		result = false;
	//------------------------ VOLUMETRIC FOG -------------------------//
	if(!strstr(szGLExtensions, "GL_EXT_fog_coord"))
		result = false;

	return result;
}


/****************************************************************************
 *                                                                          *
 *    Function     : init                                                   * 
 *                                                                          *
 *    Description  : Initialise les bibliothques                           *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value : Retourne vrai si russi sinon retourne faux            *
 *                                                                          *
 ****************************************************************************/
bool CGameEngine::init(void)
{
	return (initSDL() && initOGL() && initFMOD() && initDEVIL() );
}


/****************************************************************************
 *                                                                          *
 *    Function     : initSDL                                                * 
 *                                                                          *
 *    Description  : Initialise la SDL                                      *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value : Retourne vrai si russi sinon retourne faux            *
 *                                                                          *
 ****************************************************************************/
bool CGameEngine::initSDL(void)
{
	if( SDL_Init( SDL_INIT_VIDEO ) < 0 )
	{
		/* ECHEC */
		return false;
	}

    m_iVideoFlags    = SDL_OPENGL;
    m_iVideoFlags   |= SDL_HWPALETTE;
	if (m_bResizable)
		m_iVideoFlags   |= SDL_RESIZABLE;

	// MODE FULLSCREEN
	if (m_bFullScreen)
		m_iVideoFlags   |=SDL_FULLSCREEN;

    const SDL_VideoInfo * videoInfo = SDL_GetVideoInfo();
 	if( !videoInfo ) 
	{
		/* ECHEC */
		return false;
	}
	m_iScreenBpp = videoInfo->vfmt->BitsPerPixel;

	if(videoInfo -> hw_available)
	{ 
        m_iVideoFlags |= SDL_HWSURFACE;
    }
    else
    { 
        m_iVideoFlags |= SDL_SWSURFACE;
    }

    if(videoInfo -> blit_hw)
    {
        m_iVideoFlags |= SDL_HWACCEL;
    }
    else
    {
    }

	if ( !createWindow())
		return false;

	// Hollowed Cursor
	SDL_ShowCursor(0);

	return true;
}


/****************************************************************************
 *                                                                          *
 *    Function     : initFMOD                                               * 
 *                                                                          *
 *    Description  : Initializes FMOD library (sound)                       *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value : Returns true if succeed, else returns false            *
 *                                                                          *
 ****************************************************************************/
bool CGameEngine::initFMOD(void)
{
	CSoundManager *pSound = CSoundManager::GetInstance();
    if (!pSound->Init())
	{
		return false;
	}

	return true;
}


/****************************************************************************
 *                                                                          *
 *    Function     : initDEVIL                                              * 
 *                                                                          *
 *    Description  : Initializes DEVIL (images loader - ie OpenIL)          *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value : Returns true if succeed, else returns false            *
 *                                                                          *
 ****************************************************************************/
bool CGameEngine::initDEVIL(void)
{
	ilInit();
	iluInit();
	
	ilEnable(IL_ORIGIN_SET);
	ilEnable(IL_FORMAT_SET);
	ilEnable(IL_TYPE_SET);

	ilutEnable(ILUT_OPENGL_CONV);
	ilOriginFunc(IL_ORIGIN_LOWER_LEFT);
	ilTypeFunc(IL_UNSIGNED_BYTE);
	ilFormatFunc(IL_BGR);
	ilEnable(IL_FILE_OVERWRITE);
	ilSetInteger(IL_FORMAT_MODE, IL_BGRA);

	ilutRenderer(ILUT_OPENGL);

	return true;
}


/****************************************************************************
 *                                                                          *
 *    Function     : initOGL                                                * 
 *                                                                          *
 *    Description  : Initializes OpenGL area                                *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
bool CGameEngine::initOGL(void)
{
	reshape(s_iScreenWidth,s_iScreenHeight);

	glClearColor(0.0f,0.0f,0.0f,0.5f);
	glClearDepth(1.0f);
	glDepthFunc(GL_LEQUAL);
	glEnable(GL_DEPTH_TEST);
	glShadeModel(GL_SMOOTH);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);

	GLenum err = glewInit();
	if (GLEW_OK != err)
	{
		/* Problem: glewInit failed, something is seriously wrong. */
		fprintf(stderr, "Error: %s\n", glewGetErrorString(err));
		return false;
	}

	return true;
}


/****************************************************************************
 *                                                                          *
 *    Function     : exec                                                   * 
 *                                                                          *
 *    Description  : Main loop of the game automaton                        *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value : Returns 0 if done width succeed or 1 if failed         *
 *                                                                          *
 ****************************************************************************/
int CGameEngine::exec(void)
{
	if (m_bInited)
	{
		m_bRunning = true;
		m_oCurrentProcess = getProcess(CBaseProcess::LoadingProcess);
		m_oCurrentProcess->init();
		m_oCurrentProcess->start();

		m_iLastCount = getTime();
		while (m_bRunning)
		{
			checkProcess();
			checkEvents();
			m_iTickCount = getTime();
			update(m_iTickCount-m_iLastCount);
			m_iLastCount = m_iTickCount;
			paint();
		}
	}
	else
	{
		return 1;
	}

	return 0;
}


/****************************************************************************
 *                                                                          *
 *    Function     : terminate                                              * 
 *                                                                          *
 *    Description  : Termines engine                                        *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CGameEngine::terminate(void)
{
	SDL_Quit( );
}


/****************************************************************************
 *                                                                          *
 *    Function     : update                                                 * 
 *                                                                          *
 *    Description  : Updates the current proccess                           *
 *                                                                          *
 *    Interface    : I time = elapsed time from the last automaton cycle    *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CGameEngine::update(Uint32 time)
{
	if (m_oCurrentProcess)
		m_oCurrentProcess->update(time);
}


/****************************************************************************
 *                                                                          *
 *    Function     : createWindow                                           * 
 *                                                                          *
 *    Description  : Creates the graphic user interface                     *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value : Returns true if succeed, else returns false            *
 *                                                                          *
 ****************************************************************************/
bool CGameEngine::createWindow()
{
	SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );
    SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE,   m_iScreenBpp); 
    SDL_GL_SetAttribute( SDL_GL_STENCIL_SIZE, 0);
    SDL_GL_SetAttribute( SDL_GL_ACCUM_RED_SIZE, 0);
    SDL_GL_SetAttribute( SDL_GL_ACCUM_GREEN_SIZE, 0);
    SDL_GL_SetAttribute( SDL_GL_ACCUM_BLUE_SIZE, 0);
    SDL_GL_SetAttribute( SDL_GL_ACCUM_ALPHA_SIZE, 0);

	SDL_GL_SetAttribute(SDL_GL_RED_SIZE,  16);
    SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, 16);
    SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE,  16);

    SDL_GL_SetAttribute(SDL_GL_ALPHA_SIZE, 16);
    SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 32);

	ecran = SDL_SetVideoMode( s_iScreenWidth, s_iScreenHeight, m_iScreenBpp, m_iVideoFlags );
	if( ecran == 0 )
	{
		return false;
	}

	return true;
}


/****************************************************************************
 *                                                                          *
 *    Function     : reshape                                                * 
 *                                                                          *
 *    Description  : Reshapes GUI                                           *
 *                                                                          *
 *    Interface    : I width  = screen width                                *
 *                   I height = screen height                               * 
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CGameEngine::reshape(int width, int height)
{
	glViewport(0,0,(GLsizei)(width),(GLsizei)(height));						
	glMatrixMode(GL_PROJECTION);											
	glLoadIdentity();														
}


/****************************************************************************
 *                                                                          *
 *    Function     : checkProcess                                           * 
 *                                                                          *
 *    Description  : Checks next process state                              *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CGameEngine::checkProcess(void)
{
	if (m_oCurrentProcess)
	{
		switch (m_oCurrentProcess->getState())
		{
		case CBaseProcess::Running:
		case CBaseProcess::None:
			break;
		case CBaseProcess::Done:
			m_oCurrentProcess->terminate();
			switch(m_oCurrentProcess->getType())
			{
			case CBaseProcess::LoadingProcess:
				{
				CTextureManager *pTex = CTextureManager::GetInstance();
				pTex->readFile("./xml/textures.xml");
				CSoundManager *pSound = CSoundManager::GetInstance();
				pSound->readFile("./xml/sounds.xml");
				m_oCurrentProcess = getProcess(CBaseProcess::IntroProcess);
				m_oCurrentProcess->init();
				m_oCurrentProcess->start();
				}
				break;

			case CBaseProcess::IntroProcess:
				m_oCurrentProcess = getProcess(CBaseProcess::MenuProcess);
				m_oCurrentProcess->init();
				m_oCurrentProcess->start();
				break;
				
				
			case CBaseProcess::MenuProcess:
				m_oCurrentProcess = getProcess(CBaseProcess::GameProcess);
				m_oCurrentProcess->init();
				m_oCurrentProcess->start();
				break;
				
				
			case CBaseProcess::GameProcess:
				m_oCurrentProcess = getProcess(CBaseProcess::MenuProcess);
				m_oCurrentProcess->start();
				break;
				
			case CBaseProcess::EndProcess:
				m_bRunning = false;
				break;
				
				
			default:
				m_oCurrentProcess = 0;
				break;
			}
			break;

		case CBaseProcess::Paused:
			if (m_oCurrentProcess->getType() == CBaseProcess::GameProcess)
				m_oCurrentProcess = getProcess(CBaseProcess::MenuProcess);
			break;
		case CBaseProcess::Quit:
				m_bRunning = false;
			break;
		}

	}
}


/****************************************************************************
 *                                                                          *
 *    Function     : getTime                                                * 
 *                                                                          *
 *    Description  : Calculates  elapsed time since the last automaton cycle*
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value : Elapsed time in milliseconds                           *
 *                                                                          *
 ****************************************************************************/
Uint32 CGameEngine::getTime(void)
{
	return SDL_GetTicks();
}

void CGameEngine::paint(void)
{
	if (m_oCurrentProcess)
		m_oCurrentProcess->paint();

	SDL_GL_SwapBuffers( );
}


/****************************************************************************
 *                                                                          *
 *    Function     : checkEvents                                            * 
 *                                                                          *
 *    Description  : Checks mouse and keyboards events                      *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CGameEngine::checkEvents(void)
{
	SDL_Event event;
	/* On traite tous les vnement de la queue */
	while( SDL_PollEvent( &event ) ) {
		switch( event.type ) {
		case SDL_KEYDOWN:
			if (event.key.keysym.sym == SDLK_F5)
			{
				CTextureManager *pTex = CTextureManager::GetInstance();
				pTex->capture();
			}

			if (m_oCurrentProcess)
				m_oCurrentProcess->keyDownEvent(&event.key);
			break;
		case SDL_KEYUP:
			if (m_oCurrentProcess)
				m_oCurrentProcess->keyUpEvent(&event.key);
			break;
		case SDL_MOUSEMOTION:
			if (m_oCurrentProcess)
				m_oCurrentProcess->mouseMotionEvent(&event.motion);
			break;
		case SDL_MOUSEBUTTONDOWN:
			if (m_oCurrentProcess)
				m_oCurrentProcess->mouseButtonDownEvent(&event.button);
			break;
		case SDL_MOUSEBUTTONUP:
			if (m_oCurrentProcess)
				m_oCurrentProcess->mouseButtonUpEvent(&event.button);
			break;
		case SDL_QUIT:
			m_bRunning = false;;
			break;
		case SDL_VIDEORESIZE:		
			reshape(event.resize.w, event.resize.h);
			break;
		}
	}
}


/****************************************************************************
 *                                                                          *
 *    Function     : paint                                                  * 
 *                                                                          *
 *    Description  : Returns a pointer on the current proccess              *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value : Returns a pointer or 0 if not defined                  *
 *                                                                          *
 ****************************************************************************/
CBaseProcess* CGameEngine::getProcess(CBaseProcess::type _type)
{
	for (int i = 0; i<(int)m_vProcess.size();i++)
	{
		if (m_vProcess.at(i)->getType() == _type)
			return m_vProcess.at(i);
	}
	return 0;
}




/****************************************************************************
 *                                                                          *
 *  NOTES :                                                                 *
 *                                                                          *
 ****************************************************************************
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *    
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *         
 *                                                                          *    
 ****************************************************************************/ 
 