/****************************************************************************
 *                                                                          *
 *                  Concours 2006 - GAMES CREATORS NETWORK                  *
 *                                                                          *
 ****************************************************************************

                     PRODUCT          : GRAVITY GAME
 
                     FILE             : GameEngine.h

                     VERSION          : 1
 
 ****************************************************************************
 *                                                                          *
 *  DESCRIPTION :                                                           *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          * 
 *                                                                          *  
 ****************************************************************************
 *                                                                          *
 * HISTORY :                                                                * 
 * ---------                                                                *
 *                                                                          *
 *  Date        | Author  | Comment                                         *
 *  ---------------------------------------------------------------------   *
 *  24/07/2006  | Bahamut | Creation                                        *
 *  ---------------------------------------------------------------------   *
 *                                                                          *
 *                                                                          *
 ****************************************************************************/

#ifndef _GAME_ENGINE_H__
#define _GAME_ENGINE_H__ 1

#include "global.h"
#include <vector>
#include "./process/BaseProcess.h"
#include "./process/LoadingProcess.h"
#include "./process/IntroProcess.h"
#include "./process/MenuProcess.h"
#include "./process/GameProcess.h"
#include "./process/EndProcess.h"
#include "./managers/SoundManager.h"
#include "./managers/TextureManager.h"
#include "./IL/ilut.h"

class CGameEngine
{
public:
	CGameEngine(void);
	~CGameEngine(void);
	int exec(void);

	static int ScreenWidth()  { return s_iScreenWidth;  }
	static int ScreenHeight() { return s_iScreenHeight; }

protected:
	bool m_bRunning;
	bool m_bInited;
	bool m_bTexturesDebugMode;
	Uint32 m_iTickCount;
	Uint32 m_iLastCount;
	int m_iScreenBpp;
	int m_iVideoFlags;
	static int s_iScreenWidth;
	static int s_iScreenHeight;
	bool m_bResizable;
	bool m_bFullScreen;
	CBaseProcess* m_oCurrentProcess;
	std::vector<CBaseProcess *> m_vProcess;

	bool init(void);
	bool initSDL(void);
	bool initOGL(void);
	bool initFMOD(void);
	bool initDEVIL(void);
	bool checkOGLExtensions(void);
	void terminate(void);
	void update(Uint32 time);
	bool createWindow();
	void reshape(int width, int height);
	void checkProcess(void);
	Uint32 getTime(void);
	void paint(void);
	void checkEvents(void);

private:
	CBaseProcess* getProcess(CBaseProcess::type _type);
	SDL_Surface* ecran;
};

#endif



/****************************************************************************
 *                                                                          *
 *  NOTES :                                                                 *
 *                                                                          *
 ****************************************************************************
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *    
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *         
 *                                                                          *    
 ****************************************************************************/