/****************************************************************************
 *                                                                          *
 *                  Concours 2006 - GAMES CREATORS NETWORK                  *
 *                                                                          *
 ****************************************************************************

                     PRODUCT          : GRAVITY GAME
 
                     FILE             : GObject.cpp

                     VERSION          : 1
 
 ****************************************************************************
 *                                                                          *
 *  DESCRIPTION :                                                           *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          * 
 *                                                                          *  
 ****************************************************************************
 *                                                                          *
 * HISTORY :                                                                * 
 * ---------                                                                *
 *                                                                          *
 *  Date        | Author  | Comment                                         *
 *  ---------------------------------------------------------------------   *
 *  24/07/2006  | Bahamut | Creation                                        *
 *  ---------------------------------------------------------------------   *
 *                                                                          *
 *                                                                          *
 ****************************************************************************/

#include "GObject.h"
#include "../managers/SoundManager.h"

/****************************************************************************
 *                                                                          *
 *    Function     : GObject                                                * 
 *                                                                          *
 *    Description  : Class construtor                                       *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
GObject::GObject(void):
m_bVisible(true),
m_bActived(false),
m_iUniqId(0),
m_iParentId(0),
m_iNormalTextureId(0),
m_fTransparency(0.95f),
m_bSoundEnabled(false),
m_sClickSoundName(""),
m_sUnClickSoundName("")
{
	m_cNormalTexCoord.set(0.0f, 0.0f, 1.0f, 1.0f);
}


/****************************************************************************
 *                                                                          *
 *    Function     : GObject                                                * 
 *                                                                          *
 *    Description  : Class construtor                                       *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
GObject::GObject(int parentId, int id, const CPoint2D<int> &pos1, const CPoint2D<int> &pos2 )
{
	GObject( parentId, id, pos1.X(), pos1.Y(), pos2.X()-pos1.X(), pos2.Y()-pos1.Y() );
}


/****************************************************************************
 *                                                                          *
 *    Function     : GObject                                                * 
 *                                                                          *
 *    Description  : Class construtor                                       *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
GObject::GObject( int parentId, int id, int x, int y, int w, int h )
{
	type = e_none;
	m_bVisible = true;
	m_bActived = false;
	m_iNormalTextureId = 0;
	m_iUniqId = id;
	m_fTransparency = 0.7f;
	m_iParentId = parentId;
	m_cRect.set(CPoint2D<int>(x,y),CPoint2D<int>(x+w,y+h));
	m_bSoundEnabled = false;
	m_sClickSoundName = "";
	m_sUnClickSoundName = "";

	m_iNormalTextureId = 0;
	m_iDisabledTextureId = 0;
	m_cNormalTexCoord.set(0.0f, 0.0f, 1.0f, 1.0f);
	m_cDisabledTexCoord.set(0.0f, 0.0f, 1.0f, 1.0f);
}


/****************************************************************************
 *                                                                          *
 *    Function     : setDisabledTexture                                     * 
 *                                                                          *
 *    Description  :                                                        *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void GObject::setDisabledTexture(const char* textureName, CRectangle<GLfloat> &coordTexture)
{
	CTextureManager *pTex = CTextureManager::GetInstance();
	m_iDisabledTextureId = pTex->index(textureName);
	m_cDisabledTexCoord = coordTexture;
}


/****************************************************************************
 *                                                                          *
 *    Function     : setNormalTexture                                       * 
 *                                                                          *
 *    Description  :                                                        *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void GObject::setNormalTexture(const char* textureName, CRectangle<GLfloat> &coordTexture)
{
	CTextureManager *pTex = CTextureManager::GetInstance();
	m_iNormalTextureId = pTex->index(textureName);
	m_cNormalTexCoord = coordTexture;
}


/****************************************************************************
 *                                                                          *
 *    Function     : setPosition                                            *
 *                                                                          *
 *    Description  :                                                        *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void GObject::setPosition(const CPoint2D<int> &position)
{
	CPoint2D<int> pos1 = position;
	CPoint2D<int> pos2 = pos1 + CPoint2D<int>(width(), height());
	m_cRect.set( pos1 , pos2 );
}


/****************************************************************************
 *                                                                          *
 *    Function     : setPosition                                            *
 *                                                                          *
 *    Description  :                                                        *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void GObject::setPosition(int x, int y)
{
	CPoint2D<int> pos1 = CPoint2D<int>(x,y);
	CPoint2D<int> pos2 = pos1 + CPoint2D<int>(width(), height());
	m_cRect.set( pos1 , pos2 );
}


/****************************************************************************
 *                                                                          *
 *    Function     : resize                                                 *
 *                                                                          *
 *    Description  :                                                        *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void GObject::resize(int w, int h)
{
	CPoint2D<int> pos1 = m_cRect.position();
	CPoint2D<int> pos2 = pos1 + CPoint2D<int>(w, h);
}


/****************************************************************************
 *                                                                          *
 *    Function     : hasClick                                               * 
 *                                                                          *
 *    Description  :                                                        *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
bool GObject::hasClick(const CPoint2D<int> &cursor)
{
	if (m_cRect.inside(cursor))
	{
		if (m_bSoundEnabled && m_sClickSoundName!= "")
			playSound(m_sClickSoundName);

		m_bActived = true;
		return true;
	}
	m_bActived = false;
	return false;
}


/****************************************************************************
 *                                                                          *
 *    Function     : hasClick                                               * 
 *                                                                          *
 *    Description  :                                                        *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
bool GObject::hasUnClick(const CPoint2D<int> &cursor)
{
	if (m_cRect.inside(cursor))
	{
		if (m_bSoundEnabled && m_sUnClickSoundName!= "")
			playSound(m_sUnClickSoundName);

		m_bActived = false;
		return true;
	}
	m_bActived = false;
	return false;
}


/****************************************************************************
 *                                                                          *
 *    Function     : setTransparency                                        * 
 *                                                                          *
 *    Description  :                                                        *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void GObject::setTransparency(float alpha)
{
	if (alpha >= 0.0f && alpha <=1.0f)
        m_fTransparency = alpha;
}


/****************************************************************************
 *                                                                          *
 *    Function     : playSound                                              * 
 *                                                                          *
 *    Description  : Plays the sound passed in parameter                    *
 *                                                                          *
 *    Interface    : I name : the name of the sound to play                 *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void GObject::playSound(std::string name)
{
	CSoundManager *pSound = CSoundManager::GetInstance();
	pSound->PlayMusic(name);
}

