/****************************************************************************
 *                                                                          *
 *                  Concours 2006 - GAMES CREATORS NETWORK                  *
 *                                                                          *
 ****************************************************************************

                     PRODUCT          : GRAVITY GAME
 
                     FILE             : GObject.h

                     VERSION          : 1
 
 ****************************************************************************
 *                                                                          *
 *  DESCRIPTION :                                                           *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          * 
 *                                                                          *  
 ****************************************************************************
 *                                                                          *
 * HISTORY :                                                                * 
 * ---------                                                                *
 *                                                                          *
 *  Date        | Author  | Comment                                         *
 *  ---------------------------------------------------------------------   *
 *  24/07/2006  | Bahamut | Creation                                        *
 *  ---------------------------------------------------------------------   *
 *                                                                          *
 *                                                                          *
 ****************************************************************************/


#ifndef _GRAPHIC_OBJECT_H__
#define _GRAPHIC_OBJECT_H__ 1

#include <string>
#include "../utils/Rectangle.h"
#include "../managers/TextureManager.h"

// ==============================================
// GObject - Graphical object - Base of alls UI objects
// ==============================================
class GObject
{
public:

	enum {e_pushbutton, e_checkbox, e_image, e_window, e_none};

	GObject(void);
	GObject(int parentId, int id, const CPoint2D<int> &pos1, const CPoint2D<int> &pos2);
	GObject(int parentId, int id, int x, int y, int w, int h);
	virtual ~GObject(void) {}

	void setDisabledTexture(const char* textureName, CRectangle<GLfloat> &coordTexture);
	void setNormalTexture(const char* textureName, CRectangle<GLfloat> &coordTexture);
	void setVisible(bool state) { m_bVisible = state; }
	virtual void setEnabled(bool state) { m_bEnabled = state; }
	void setPosition(const CPoint2D<int> &position);
	void setPosition(int x, int y);
	void setTransparency(float alpha);
	void resize(int w, int h);
	void move(const CPoint2D<int> &position) { m_cRect.translate(position.X(), position.Y()); }
	void move(int x, int y) { m_cRect.translate(x, y); }
	void desactive() { m_bActived = 0; }

	int width() const  { return m_cRect.width();}
	int height() const { return m_cRect.height();}
	CPoint2D<int> position() const { return m_cRect.position(); }
	CRectangle<int> rect() const { return m_cRect; }
	int identifier() const { return m_iUniqId; }
	int parent() const { return m_iParentId; }
	float transparency() const { return m_fTransparency; }

	bool isVisible() { return m_bVisible; }
	bool isEnabled() { return m_bEnabled; }

	virtual bool hasClick(const CPoint2D<int> &cursor);
	virtual bool hasUnClick(const CPoint2D<int> &cursor);

	void setSoundOnClick(std::string name) { m_sClickSoundName = name; }
	void setSoundOnUnClick(std::string name) { m_sUnClickSoundName = name; }
	void setEnableSound(bool state) { m_bSoundEnabled = state; }
	void setParent(int parent) { m_iParentId = parent; }

	virtual void render() = 0;

	int type;

protected:
	void playSound(std::string name);

	int m_iUniqId;
	int m_iParentId;
	bool m_bVisible;
	bool m_bEnabled;
	bool m_bActived;
	bool m_bSoundEnabled;
	std::string m_sClickSoundName;
	std::string m_sUnClickSoundName;

	float m_fTransparency;

	GLuint m_iNormalTextureId;
	GLuint m_iDisabledTextureId;
	CRectangle<GLfloat> m_cNormalTexCoord;
	CRectangle<GLfloat> m_cDisabledTexCoord;

	CRectangle<int> m_cRect;

};


#endif


/****************************************************************************
 *                                                                          *
 *  NOTES :                                                                 *
 *                                                                          *
 ****************************************************************************
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *    
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *         
 *                                                                          *    
 ****************************************************************************/
