/****************************************************************************
 *                                                                          *
 *                  Concours 2006 - GAMES CREATORS NETWORK                  *
 *                                                                          *
 ****************************************************************************

                     PRODUCT          : GRAVITY GAME
 
                     FILE             : GPushButton.cpp

                     VERSION          : 1
 
 ****************************************************************************
 *                                                                          *
 *  DESCRIPTION :                                                           *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          * 
 *                                                                          *  
 ****************************************************************************
 *                                                                          *
 * HISTORY :                                                                * 
 * ---------                                                                *
 *                                                                          *
 *  Date        | Author  | Comment                                         *
 *  ---------------------------------------------------------------------   *
 *  24/07/2006  | Bahamut | Creation                                        *
 *  ---------------------------------------------------------------------   *
 *                                                                          *
 *                                                                          *
 ****************************************************************************/

#include "GPushButton.h"

/****************************************************************************
 *                                                                          *
 *    Function     : GPushButton                                            * 
 *                                                                          *
 *    Description  : Class construtor                                       *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
GPushButton::GPushButton(void)
{
	type = e_pushbutton;
	m_iPushedTextureId = 0;
	m_cPushedTexCoord.set(0.0f, 0.0f, 1.0f, 1.0f);
	m_pCallback = 0;
}


/****************************************************************************
 *                                                                          *
 *    Function     : GPushButton                                            * 
 *                                                                          *
 *    Description  : Class construtor                                       *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
GPushButton::GPushButton(int id, const CPoint2D<int> &pos1, const CPoint2D<int> &pos2)
:GObject(0, id, pos1, pos2)
{
	type = e_pushbutton;
	m_iPushedTextureId = 0;
	m_cPushedTexCoord.set(0.0f, 0.0f, 1.0f, 1.0f);
	m_pCallback = 0;
}


/****************************************************************************
 *                                                                          *
 *    Function     : GPushButton                                            * 
 *                                                                          *
 *    Description  : Class construtor                                       *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
GPushButton::GPushButton(int id, int x, int y, int w, int h)
:GObject(0, id, x, y, w, h)
{
	type = e_pushbutton;
	m_iPushedTextureId = 0;
	m_cPushedTexCoord.set(0.0f, 0.0f, 1.0f, 1.0f);
	m_pCallback = 0;
}


/****************************************************************************
 *                                                                          *
 *    Function     : ~GPushButton                                           * 
 *                                                                          *
 *    Description  : Class Destructor                                       *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
GPushButton::~GPushButton()
{
	if (m_pCallback) // delete callback if exists
		delete m_pCallback;
}


/****************************************************************************
 *                                                                          *
 *    Function     : initCallback                                           * 
 *                                                                          *
 *    Description  : Installs a callback                                    *
 *                                                                          *
 *    Interface    : I pCallback = new callback                             *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void GPushButton::initCallback(TCallbackBase* pCallback)
{
	if (m_pCallback) // if already existing callback, we must delete it
		delete m_pCallback;

	m_pCallback = pCallback;
}

/****************************************************************************
 *                                                                          *
 *    Function     : setDisabledTexture                                     * 
 *                                                                          *
 *    Description  :                                                        *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void GPushButton::setPushedTexture(const char* textureName, CRectangle<GLfloat> &coordTexture)
{
	CTextureManager *pTex = CTextureManager::GetInstance();
	m_iPushedTextureId = pTex->index(textureName);
	m_cPushedTexCoord = coordTexture;
}


/****************************************************************************
 *                                                                          *
 *    Function     :                                                        * 
 *                                                                          *
 *    Description  :                                                        *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
bool GPushButton::hasUnClick(const CPoint2D<int> &cursor)
{
	if (m_cRect.inside(cursor))
	{
		if (m_bActived)
			if(m_pCallback)
			{
				m_pCallback->Action(m_iParentId,m_iUniqId);
			}
		m_bActived = false;
		return true;
	}
	m_bActived = false;
	return false;
}


/****************************************************************************
 *                                                                          *
 *    Function     :                                                        * 
 *                                                                          *
 *    Description  :                                                        *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void GPushButton::render()
{	
	if (m_bVisible)
	{
		int idTexture = m_iNormalTextureId;
		CRectangle<GLfloat> coordUVTex = m_cNormalTexCoord;
		if (m_bEnabled)
		{
			if (m_bActived)
			{
				idTexture  = m_iPushedTextureId;
				coordUVTex = m_cPushedTexCoord;
			}
		}
		else
		{
			idTexture  = m_iDisabledTextureId;
			coordUVTex = m_cDisabledTexCoord;
		}

		glBindTexture(GL_TEXTURE_2D, idTexture);
		glColor4f(1.0f, 1.0f, 1.0f, m_fTransparency);

		glBegin( GL_QUADS );
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, coordUVTex.position().X(),coordUVTex.position2().Y());
				glVertex2i(m_cRect.position().X(),m_cRect.position().Y());
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, coordUVTex.position2().X(),coordUVTex.position2().Y());
				glVertex2i(m_cRect.position2().X(),m_cRect.position().Y());
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, coordUVTex.position2().X(),coordUVTex.position().Y());
				glVertex2i(m_cRect.position2().X(),m_cRect.position2().Y());
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, coordUVTex.position().X(),coordUVTex.position().Y());
				glVertex2i(m_cRect.position().X(),m_cRect.position2().Y());
		glEnd();
	}
}