/****************************************************************************
 *                                                                          *
 *                  Concours 2006 - GAMES CREATORS NETWORK                  *
 *                                                                          *
 ****************************************************************************

                     PRODUCT          : GRAVITY GAME
 
                     FILE             : GPushButton.h

                     VERSION          : 1
 
 ****************************************************************************
 *                                                                          *
 *  DESCRIPTION :                                                           *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          * 
 *                                                                          *  
 ****************************************************************************
 *                                                                          *
 * HISTORY :                                                                * 
 * ---------                                                                *
 *                                                                          *
 *  Date        | Author  | Comment                                         *
 *  ---------------------------------------------------------------------   *
 *  24/07/2006  | Bahamut | Creation                                        *
 *  ---------------------------------------------------------------------   *
 *                                                                          *
 *                                                                          *
 ****************************************************************************/
#ifndef _GRAPHIC_PUSH_BUTTON_H__
#define _GRAPHIC_PUSH_BUTTON_H__ 1

#include "GObject.h"


// ==============================================
// TCallbackBase - Generic callback
// ==============================================
class TCallbackBase
{
public:
    virtual void Action( int window, int button ) = 0;
};


// ==============================================
// GCallback - Callback
// ==============================================
template<class C>
class GCallback: public TCallbackBase
{
public:
    GCallback()      { m_pClass = 0; m_pAction = 0; }
    typedef void (C::*Action_t)(int window, int button);
    void SetClass(C* pClass)			{ m_pClass = pClass; }
    void SetAction(Action_t pAction)	{ m_pAction = pAction; }

protected:
    void Action(int window, int button)
    {
        if(m_pClass && m_pAction)
            (m_pClass->*m_pAction) (window, button);
    }

private:
    C*			m_pClass;
    Action_t	m_pAction;
};


// ==============================================
// GPushButton - Push Button for user interface
// ==============================================
class GPushButton : public GObject
{
public:
	GPushButton(void);
	GPushButton(int id, const CPoint2D<int> &pos1, const CPoint2D<int> &pos2);
	GPushButton(int id, int x, int y, int w, int h);
	virtual ~GPushButton(void);

	//void (*action2)();
	//void (*action)( int window, int button );
	void initCallback(TCallbackBase* pCallback);
	void setPushedTexture(const char* textureName, CRectangle<GLfloat> &coordTexture);
	void setPushed(bool state) { m_bActived = state; }

	bool isPushed() { return m_bActived; }

	bool hasUnClick(const CPoint2D<int> &cursor);
	virtual void render();

protected:
	GLuint m_iPushedTextureId;
	CRectangle<GLfloat> m_cPushedTexCoord;
	TCallbackBase* m_pCallback;
};

#endif



/****************************************************************************
 *                                                                          *
 *  NOTES :                                                                 *
 *                                                                          *
 ****************************************************************************
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *    
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *         
 *                                                                          *    
 ****************************************************************************/
