/****************************************************************************
 *                                                                          *
 *                  Concours 2006 - GAMES CREATORS NETWORK                  *
 *                                                                          *
 ****************************************************************************

                     PRODUCT          : GRAVITY GAME
 
                     FILE             : SoundManager.h

                     VERSION          : 1
 
 ****************************************************************************
 *                                                                          *
 *  DESCRIPTION :                                                           *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          * 
 *                                                                          *  
 ****************************************************************************
 *                                                                          *
 * HISTORY :                                                                * 
 * ---------                                                                *
 *                                                                          *
 *  Date        | Author  | Comment                                         *
 *  ---------------------------------------------------------------------   *
 *  24/07/2006  | Bahamut | Creation                                        *
 *  ---------------------------------------------------------------------   *
 *                                                                          *
 *                                                                          *
 ****************************************************************************/


#ifndef _SOUND_MANAGER_H__
#define _SOUND_MANAGER_H__ 1
 
#include <stdio.h>
#include <string>
#include <map>
#include "FMOD/fmod.h"
#include "FMOD/fmod_errors.h"
#include "TINYXML/tinyxml.h"
#include "../utils/singleton.h"

 
// ==============================================
// CSoundManager
// ==============================================
class CSoundManager : public CSingleton<CSoundManager>
{
	friend class CSingleton<CSoundManager>;
 
private:
   	CSoundManager(void);
	~CSoundManager(void);
    std::map<std::string, FSOUND_SAMPLE *>  m_tSoundTable;
    std::map<std::string, FSOUND_STREAM *>  m_tMusicTable;
 
public:
	bool Init(void);
	void Close(void);
	int LoadSample(const char * filename, const char * name);
	void PlaySample(std::string sound, bool loop = false);
	void UnloadSample(std::string sound);
	void UnloadSamples();
	void UnloadMusic(std::string music);
	void UnloadMusics();
	void stopSamples(void);
	int LoadMusic(const char * filename, const char * name);
	void PlayMusic(std::string music, bool loop = false);
	void PauseMusic();
	void ResumeMusic();
	void StopMusic();
	void SetVolume(int volume);
	int GetVolume(){return m_iCurrentVolume;}
	void SetAudioFrequency(int frequency){m_iAudioFrequency = frequency;}
	void SetAudioFormat(int format){m_iAudioFormat = format;}
	void SetAudioChannelsNumber(int channels){m_iAudioChannelsNumber = channels;}
	bool readFile(std::string fileName);
    int GetError();
	
	enum error {
		SND_NO_ERROR=0,
		SND_ERROR_WRONG_VERSION,
		SND_ERROR_INIT,
		SND_ERROR_FILE_ERROR,
		SND_ERROR_NOT_INITED,
		SND_ERROR_FILE_MISSING
	};
 
private:
	bool m_bInited;				
	bool m_iLoopingMusic;		
	int m_iAudioFrequency;		
	int m_iAudioFormat;			
	int m_iAudioChannelsNumber;
	CSoundManager::error m_eError;
	FSOUND_STREAM * m_pMusicStream;
	int m_iMusicChannel;
	int m_iCurrentVolume;
}; 
 
 #endif

 
/****************************************************************************
 *                                                                          *
 *  NOTES :                                                                 *
 *                                                                          *
 ****************************************************************************
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *    
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *         
 *                                                                          *    
 ****************************************************************************/