/****************************************************************************
 *                                                                          *
 *                  Concours 2006 - GAMES CREATORS NETWORK                  *
 *                                                                          *
 ****************************************************************************

                     PRODUCT          : GRAVITY GAME
 
                     FILE             : PhysicEngine.h

                     VERSION          : 1
 
 ****************************************************************************
 *                                                                          *
 *  DESCRIPTION :                                                           *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          * 
 *                                                                          *  
 ****************************************************************************
 *                                                                          *
 * HISTORY :                                                                * 
 * ---------                                                                *
 *                                                                          *
 *  Date        | Author  | Comment                                         *
 *  ---------------------------------------------------------------------   *
 *  25/07/2006  | Bahamut | Creation                                        *
 *  ---------------------------------------------------------------------   *
 *                                                                          *
 *                                                                          *
 ****************************************************************************/


#ifndef _PHYSIC_ENGINE_H__
#define _PHYSIC_ENGINE_H__ 1

#include <string>
#include <map>

#include "ODE/ode.h"
#include "PObject.h"

// ==============================================
// CPhysicEngine - ODE ENGINE 
// ==============================================
class CPhysicEngine
{
public:
	CPhysicEngine(void);
	~CPhysicEngine(void);

	bool Initialize(void);
	void Close(void);
	void Update(void);
	void Clean();

	void SetGravity(float x, float y, float z);
	void SetPrecisionERP(float value);
	void SetCorrectionCFM(float value);
	int CreateSphere(bool isStatic, float radius);
	int CreateBox(bool isStatic, float x, float y, float z);
	int CreatePlane(float a, float b, float c, float d);
	int CreateCylinder(bool isStatic, float radius, float length);
	void SetPosition(int id, float x, float y, float z);
	void SetRotation(int id, float angle, float x, float y, float z);
	void SetMass(int id, float mass);
	void AddForce(int id, float x, float y, float z);
	void SetForce(int id, float x, float y, float z);
	void AddTorque(int id, float x, float y, float z);
	void SetTorque(int id, float x, float y, float z);
	void SetEnabledGravity(int id, bool enable);

	PObject* Object(int id);
	void NearCallback(void *data,dGeomID o1, dGeomID o2);
	static CPhysicEngine* m_pEngine;

	void Draw(void);


protected:
	int GenerateObjectIndex();
	bool m_bInited;
	dSpaceID m_iSpace;
	dWorldID m_iWorld;
	dJointGroupID m_iJointGroup;

	int m_iCurrentIndexGenerator;
	std::map<int,PObject*> m_vObjects;

};




#endif



/****************************************************************************
 *                                                                          *
 *  NOTES :                                                                 *
 *                                                                          *
 ****************************************************************************
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *    
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *         
 *                                                                          *    
 ****************************************************************************/
