/****************************************************************************
 *                                                                          *
 *                  Concours 2006 - GAMES CREATORS NETWORK                  *
 *                                                                          *
 ****************************************************************************

                     PRODUCT          : GRAVITY GAME
 
                     FILE             : BaseProcess.cpp

                     VERSION          : 1
 
 ****************************************************************************
 *                                                                          *
 *  DESCRIPTION :                                                           *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          * 
 *                                                                          *  
 ****************************************************************************
 *                                                                          *
 * HISTORY :                                                                * 
 * ---------                                                                *
 *                                                                          *
 *  Date        | Author  | Comment                                         *
 *  ---------------------------------------------------------------------   *
 *  24/07/2006  | Bahamut | Creation                                        *
 *  ---------------------------------------------------------------------   *
 *                                                                          *
 *                                                                          *
 ****************************************************************************/

#include "BaseProcess.h"


/****************************************************************************
 *                                                                          *
 *    Function     : CBaseProcess                                           * 
 *                                                                          *
 *    Description  : Class construtor                                       *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
CBaseProcess::CBaseProcess(void)
{
	m_eType = NoProcess;
	m_eState = None;
}


/****************************************************************************
 *                                                                          *
 *    Function     : ~CBaseProcess                                          * 
 *                                                                          *
 *    Description  : Class Destrutor                                        *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
CBaseProcess::~CBaseProcess(void)
{
}


/****************************************************************************
 *                                                                          *
 *    Function     : init                                                   * 
 *                                                                          *
 *    Description  : Initializes                                            *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value : Returns true if succeed, else returns false            *
 *                                                                          *
 ****************************************************************************/
bool CBaseProcess::init(void)
{
	//TO REDEFINE
	return false;
}


/****************************************************************************
 *                                                                          *
 *    Function     : paint                                                  * 
 *                                                                          *
 *    Description  : Renders                                                *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CBaseProcess::paint(void)
{
	//TO REDEFINE
}


/****************************************************************************
 *                                                                          *
 *    Function     : update                                                 * 
 *                                                                          *
 *    Description  : Updates the scene                                      *
 *                                                                          *
 *    Interface    : I time = elapsed time since the last automaton cycle   *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CBaseProcess::update(Uint32 time)
{
	//TO REDEFINE
}


/****************************************************************************
 *                                                                          *
 *    Function     : terminate                                              * 
 *                                                                          *
 *    Description  : Terminates the process                                 *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CBaseProcess::terminate(void)
{
	//TO REDEFINE
}


/****************************************************************************
 *                                                                          *
 *    Function     : start                                                  * 
 *                                                                          *
 *    Description  : Starts process                                         *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CBaseProcess::start(void)
{
	m_eState = CBaseProcess::Running;
	initOGL();
	initGameState();
}


/****************************************************************************
 *                                                                          *
 *    Function     : getType                                                * 
 *                                                                          *
 *    Description  : Gives the process type                                 *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value : Returns the type of the process                        *
 *                                                                          *
 ****************************************************************************/
const CBaseProcess::type CBaseProcess::getType(void)
{
	return m_eType;
}


/****************************************************************************
 *                                                                          *
 *    Function     : getState                                               * 
 *                                                                          *
 *    Description  : Gives the process state                                *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value : Returns the state of the process                        *
 *                                                                          *
 ****************************************************************************/
const CBaseProcess::state CBaseProcess::getState(void)
{
	return m_eState;
}


/****************************************************************************
 *                                                                          *
 *    Function     : keyDownEvent                                           * 
 *                                                                          *
 *    Description  : Key down traitment                                     *
 *                                                                          *
 *    Interface    : I key = key down                                       *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CBaseProcess::keyDownEvent(SDL_KeyboardEvent* key)
{
	//TO REDEFINE
}


/****************************************************************************
 *                                                                          *
 *    Function     : keyUpEvent                                             * 
 *                                                                          *
 *    Description  : Key up traitment                                       *
 *                                                                          *
 *    Interface    : I key = key up                                         *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CBaseProcess::keyUpEvent(SDL_KeyboardEvent* key)
{
	//TO REDEFINE
}


/****************************************************************************
 *                                                                          *
 *    Function     : mouseButtonDownEvent                                   * 
 *                                                                          *
 *    Description  : Mouse button down traitment                            *
 *                                                                          *
 *    Interface    : I button = button detected                             *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CBaseProcess::mouseButtonDownEvent(SDL_MouseButtonEvent* button)
{
	//TO REDEFINE
}


/****************************************************************************
 *                                                                          *
 *    Function     : mouseButtonUpEvent                                     * 
 *                                                                          *
 *    Description  : Mouse button up traitment                              *
 *                                                                          *
 *    Interface    : I button = button detected                             *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CBaseProcess::mouseButtonUpEvent(SDL_MouseButtonEvent* button)
{
	//TO REDEFINE
}


/****************************************************************************
 *                                                                          *
 *    Function     : mouseMotionEvent                                       * 
 *                                                                          *
 *    Description  : Mouse motion traitment                                 *
 *                                                                          *
 *    Interface    : I motion = motion description                          *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CBaseProcess::mouseMotionEvent(SDL_MouseMotionEvent* motion)
{
	//TO REDEFINE
}


/****************************************************************************
 *                                                                          *
 *    Function     : initOGL                                                * 
 *                                                                          *
 *    Description  : Initializes OpenGL area                                *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CBaseProcess::initOGL(void)
{
	//TO REDEFINE
}


/****************************************************************************
 *                                                                          *
 *    Function     : initGameState                                          * 
 *                                                                          *
 *    Description  : Initializes the game state                             *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CBaseProcess::initGameState(void)
{
	//TO REDEFINE
}




/****************************************************************************
 *                                                                          *
 *  NOTES :                                                                 *
 *                                                                          *
 ****************************************************************************
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *    
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *         
 *                                                                          *    
 ****************************************************************************/ 
 
