/****************************************************************************
 *                                                                          *
 *                  Concours 2006 - GAMES CREATORS NETWORK                  *
 *                                                                          *
 ****************************************************************************

                     PRODUCT          : GRAVITY GAME
 
                     FILE             : GameProcess.cpp

                     VERSION          : 1
 
 ****************************************************************************
 *                                                                          *
 *  DESCRIPTION :                                                           *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          * 
 *                                                                          *  
 ****************************************************************************
 *                                                                          *
 * HISTORY :                                                                * 
 * ---------                                                                *
 *                                                                          *
 *  Date        | Author  | Comment                                         *
 *  ---------------------------------------------------------------------   *
 *  24/07/2006  | Bahamut | Creation                                        *
 *  ---------------------------------------------------------------------   *
 *                                                                          *
 *                                                                          *
 ****************************************************************************/

#include "GameProcess.h"
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "../GameEngine.h"


#define LEVEL_NUMBER	5

/****************************************************************************
 *                                                                          *
 *    Function     : CGameProcess                                           * 
 *                                                                          *
 *    Description  : Class construtor                                       *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
CGameProcess::CGameProcess(void)
{
	m_eType = GameProcess;
	m_eGameState = loadingLevel;
	m_iCurrentLvlNb = 0;
	m_bLeaveQuestion = false;
	m_bReadyToLoad = false;
	m_bLoaded = false;
	m_bPaused = false;
	m_iTimePast = 0;
	m_iBallNumber = 0;
	m_bBallInHand = false;
	m_iWinningHoleId = 0;

	p_yesButton = 0;
	p_NoButton = 0;

	m_cPhysic = 0;

	m_fU = 0.0f;
	m_fV = 0.0f;

	//initialize random generator
	srand ( time(0) );
}


/****************************************************************************
 *                                                                          *
 *    Function     : ~CGameProcess                                          * 
 *                                                                          *
 *    Description  : Class Destrutor                                        *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
CGameProcess::~CGameProcess(void)
{
	if (p_yesButton) delete p_yesButton;
	if (p_NoButton) delete p_NoButton;
	if (m_cPhysic) delete m_cPhysic;
}


/****************************************************************************
 *                                                                          *
 *    Function     : init                                                   * 
 *                                                                          *
 *    Description  : Initializes                                            *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value : Returns true if succeed, else returns false            *
 *                                                                          *
 ****************************************************************************/
bool CGameProcess::init(void)
{
	m_eGameState = loadingLevel;
	m_iCurrentLvlNb = 0;
	m_bLeaveQuestion = false;
	m_bReadyToLoad = false;
	m_bLoaded = false;
	m_bPaused = false;
	m_iTimePast = 0;
	m_iBallNumber = 0;
	m_bBallInHand = false;
	m_iWinningHoleId = 0;

	p_yesButton = 0;
	p_NoButton = 0;

	std::map<int,CHole*> m_vHoles;
	std::map<int,CGravitationalHole*> m_vGravitationalHoles;
	std::map<int,CAnimation*> m_vHoleAnimation;

	m_fU = 0.0f;
	m_fV = 0.0f;


	m_bDrawArrow = true;

	// Precalculated coordonates for mapping (textures)
	float wSize = 512.0f;       // texture width 
	float hsize = 512.0f;       // texture height
	float ul = 175.0f/wSize;    // long button length
	float vl = 40.0f/hsize;     // long button height
	float us = 100.0f/wSize;    // short button length
	float vs = 40.0f/hsize;     // short button height
	float u0 = 0.0f;            // u coordonates first column
	float u1 = 200.0f/512.0f;   // u coordonates second column
	float u2 = 400.0f/512.0f;   // u coordonates 3th column
	float vi = 50.0f/hsize;     // v coordonates index 

	GCallback<CGameProcess>* pCallback;
	////////////// BUTTON YES //////////////
	p_yesButton = new GPushButton(0,512+20,350,170,50);
	p_yesButton->setNormalTexture("menu_buttons",CRectangle<GLfloat>(u2,vi*0,u2+us,vs+vi*0));
	p_yesButton->setPushedTexture("menu_buttons",CRectangle<GLfloat>(u2,vi*1,u2+us,vs+vi*1));
    pCallback = new GCallback<CGameProcess>;   // Create Callback
	pCallback->SetClass(this);  
	pCallback->SetAction(actionQuit);
	p_yesButton->initCallback(pCallback);  // and attach Callback to button

	////////////// BUTTON NO  //////////////
	p_NoButton    = new GPushButton(1,512-190,350,170,50);
	p_NoButton->setNormalTexture("menu_buttons",CRectangle<GLfloat>(u2,vi*2,u2+us,vs+vi*2));
	p_NoButton->setPushedTexture("menu_buttons",CRectangle<GLfloat>(u2,vi*3,u2+us,vs+vi*3));
	pCallback = new GCallback<CGameProcess>;
	pCallback->SetClass(this);  
	pCallback->SetAction(actionCancel);
	p_NoButton->initCallback(pCallback);

	CTextureManager *pTex = CTextureManager::GetInstance();
	m_iTestTexture = pTex->index("screen_test");
	m_iBallTexture = pTex->index("ball");
	m_iHandTexture = pTex->index("hand");
	m_iholeTexture = pTex->index("hole");
	m_iEndingTexture = pTex->index("endgame");
	m_iPausingTexture = pTex->index("pausegame");
	m_iNumbersTexture = pTex->index("numbers");
	m_iArrowTexture = pTex->index("arrow"); 

	// Cursor
	m_cCursor.load("menu_cursor");
	m_cCursor.setTransparency(1.0);

	// Engine
	m_cPhysic = new CPhysicEngine();
	m_cPhysic->Initialize();
	m_cPhysic->SetGravity(0.0f,-9.0f,0.0f);

	//m_cHoleAnimation = new CAnimation(500.0f, 300.0f, -20.0f, 100.0f,100.0f, "hole", 24);

	//float x, float y, float z, int width, int height, std::string texName, int number)

	m_vFireAnimation[0] = new CAnimation(120.0f, 0.0f, 0.0f, 200.0f,250.0f, "fire", 23);
	m_vFireAnimation[1] = new CAnimation(272.0f, 0.0f, 0.0f, 200.0f,250.0f, "fire", 23);
	m_vFireAnimation[2] = new CAnimation(418.0f, 0.0f, 0.0f, 200.0f,250.0f, "fire", 23);


	nextLevel();

	return true;
}


/****************************************************************************
 *                                                                          *
 *    Function     : paint                                                  * 
 *                                                                          *
 *    Description  : Renders                                                *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CGameProcess::paint(void)
{
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
	glLoadIdentity();

	gluLookAt(512.0f, 384.0f, 667.0f, 512.0f, 384.0f, 0.0f, 0.0f, 1.0f, 0.0f);

	/////////////////////////////  3D MODE  //////////////////////////////////////

	glActiveTextureARB( GL_TEXTURE1_ARB );
    glDisable(GL_TEXTURE_2D);
	glActiveTextureARB( GL_TEXTURE2_ARB );
    glDisable(GL_TEXTURE_2D);
	glActiveTextureARB( GL_TEXTURE3_ARB );
    glDisable(GL_TEXTURE_2D);
	glActiveTextureARB( GL_TEXTURE0_ARB );
    glEnable(GL_TEXTURE_2D);

	switch(m_eGameState)
	{
	case test:
		glBindTexture(GL_TEXTURE_2D, m_iTestTexture);
		glBegin(GL_QUADS);
			glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f,0.0f);glVertex3f(0.0f,768.0f,0.0f);
			glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f,0.75f);glVertex3f(0.0f,0.0f,0.0f);
			glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f,0.75f);glVertex3f(1024.0f,0.0f,0.0f);
			glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f,0.0f);glVertex3f(1024.0f,768.0f,0.0f);
		glEnd();
		break;
	case loadingLevel:   //////////////////    LOADING    LEVEL   //////////////////
		glBindTexture(GL_TEXTURE_2D, m_iLoadingTexture);
		glDisable(GL_TEXTURE_GEN_S);
		glDisable(GL_TEXTURE_GEN_T);

		glBegin(GL_QUADS);
			// Minimap
			glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f,0.0f);glVertex3f(412.0f,700.0f,0.0f);
			glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f,1.0f);glVertex3f(412.0f,400.0f,0.0f);
			glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.73f,1.0f);glVertex3f(612.0f,400.0f,0.0f);
			glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.73f,0.0f);glVertex3f(612.0f,700.0f,0.0f);
			// Level number
			glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.75f,0.99f);glVertex3f(412.0f,320.0f,0.0f);
			glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.99f,0.99f);glVertex3f(412.0f,250.0f,0.0f);
			glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.99f,0.6f);glVertex3f(612.0f,250.0f,0.0f);
			glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.75f,0.6f);glVertex3f(612.0f,320.0f,0.0f);
			// Level Name
			glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.75f,0.6f);glVertex3f(312.0f,170.0f,0.0f);
			glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.99f,0.6f);glVertex3f(312.0f,100.0f,0.0f);
			glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.99f,0.01f);glVertex3f(712.0f,100.0f,0.0f);
			glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.75f,0.01f);glVertex3f(712.0f,170.0f,0.0f);
		glEnd();
		break;
	case waiting:   //////////////////        WAITING       //////////////////
	case lostGame:  //////////////////       GAME OVER       //////////////////
	case winGame:   //////////////////        WINNER         //////////////////
	case playing:   //////////////////        PLAYING        //////////////////
		{

			// Background
			glBindTexture(GL_TEXTURE_2D, m_iBackgroundTexture);
			glBegin(GL_QUADS);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, m_fU,m_fV);glVertex3f(-50.0f,720.0f,-120.0f);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, m_fU,m_fV+1.0f);glVertex3f(-50.0f,-100.0f,-120.0f);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, m_fU+1.0f,m_fV+1.0f);glVertex3f(800.0f,-100.0f,-120.0f);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, m_fU+1.0f,m_fV);glVertex3f(800.0f,720.0f,-120.0f);
			glEnd();

			glEnable(GL_BLEND);

			// Foreground
			glBindTexture(GL_TEXTURE_2D, m_iForegroundTexture);
			glBegin(GL_QUADS);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f,0.0f);glVertex3f(0.0f,768.0f,-5.0f);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f,0.75f);glVertex3f(0.0f,0.0f,-5.0f);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f,0.75f);glVertex3f(1024.0f,0.0f,-5.0f);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f,0.0f);glVertex3f(1024.0f,768.0f,-5.0f);
			glEnd();

			// the arrow
			if (m_bDrawArrow)
			{
				int xArrow = (m_vHoles[m_iWinningHoleId]->xmax + m_vHoles[m_iWinningHoleId]->xmin)/2;
				glBindTexture(GL_TEXTURE_2D, m_iArrowTexture);
				glBegin(GL_QUADS);
					glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f,0.0f);glVertex3f(xArrow-25.0f,80.0f,25.0f);
					glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f,1.0f);glVertex3f(xArrow-25.0f,20.0f,25.0f);
					glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f,1.0f);glVertex3f(xArrow+25.0f,20.0f,25.0f);
					glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f,0.0f);glVertex3f(xArrow+25.0f,80.0f,25.0f);
				glEnd();
			}

			// Draw Gravitational Holes
			std::map<int,CAnimation*>::const_iterator itr;  // m_vHoleAnimation;
			for(itr = m_vHoleAnimation.begin(); itr != m_vHoleAnimation.end(); ++itr)
			{
				(*itr).second->Draw();
			}

			// Draws flames on Bahamut effect
			if (m_iEffectMode == GFX_BAHAMUT)
			{
				m_vFireAnimation[0]->Draw();
				m_vFireAnimation[1]->Draw();
				m_vFireAnimation[2]->Draw();
			}

			PObject* obj = m_cPhysic->Object(mi_BallId);
			float x = obj->x;
			float y = obj->y;
			float z = obj->z;

			if (m_bBallInHand)
			{
				// Draws the hand
				glBindTexture(GL_TEXTURE_2D, m_iHandTexture);
				glBegin(GL_QUADS);
					glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f,0.0f);glVertex3f(x-163.0f,y+80.0f,0.0f);
					glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f,1.0f);glVertex3f(x-163.0f,y-20.0f,0.0f);
					glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f,1.0f);glVertex3f(x+32.0f,y-20.0f,0.0f);
					glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f,0.0f);glVertex3f(x+32.0f,y+80.0f,0.0f);
				glEnd();
			}

			// THE BALL
			glBindTexture(GL_TEXTURE_2D, m_iBallTexture);
				glBegin(GL_QUADS);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f,0.0f);glVertex3f(x-10.0f,y+10.0f,0.0f);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f,1.0f);glVertex3f(x-10.0f,y-10.0f,0.0f);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f,1.0f);glVertex3f(x+10.0f,y-10.0f,0.0f);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f,0.0f);glVertex3f(x+10.0f,y+10.0f,0.0f);
			glEnd();

			// STATIC THE BALL FOR SCORE
			glBindTexture(GL_TEXTURE_2D, m_iBallTexture);
				glBegin(GL_QUADS);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f,0.0f);glVertex3f(800.0f-20.0f,536.0f+20.0f,10.0f);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f,1.0f);glVertex3f(800.0f-20.0f,536.0f-20.0f,10.0f);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f,1.0f);glVertex3f(800.0f+20.0f,536.0f-20.0f,10.0f);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f,0.0f);glVertex3f(800.0f+20.0f,536.0f+20.0f,10.0f);
			glEnd();

			// NUMBER OF BALLS
			if (m_iBallNumber<100 &&m_iBallNumber>-1)
			{
				int secondN = m_iBallNumber/10;
				int firstN = m_iBallNumber-secondN*10;

				float u = 0.0f;
				float v = 0.0f;
				findUVNumber(secondN, u, v);

				glBindTexture(GL_TEXTURE_2D, m_iNumbersTexture);
				glBegin(GL_QUADS);
					// Second Number
					glMultiTexCoord2fARB(GL_TEXTURE0_ARB, u,v);glVertex3f(867.0f,580.0f,0.0f);
					glMultiTexCoord2fARB(GL_TEXTURE0_ARB, u,v+0.25f);glVertex3f(867.0f,517.0f,0.0f);
					glMultiTexCoord2fARB(GL_TEXTURE0_ARB, u+0.25f,v+0.25f);glVertex3f(930.0f,517.0f,0.0f);
					glMultiTexCoord2fARB(GL_TEXTURE0_ARB, u+0.25f,v);glVertex3f(930.0f,580.0f,0.0f);

					findUVNumber(firstN, u, v);
					glMultiTexCoord2fARB(GL_TEXTURE0_ARB, u,v);glVertex3f(935.0f,580.0f,0.0f);
					glMultiTexCoord2fARB(GL_TEXTURE0_ARB, u,v+0.25f);glVertex3f(935.0f,517.0f,0.0f);
					glMultiTexCoord2fARB(GL_TEXTURE0_ARB, u+0.25f,v+0.25f);glVertex3f(998.0f,517.0f,0.0f);
					glMultiTexCoord2fARB(GL_TEXTURE0_ARB, u+0.25f,v);glVertex3f(998.0f,580.0f,0.0f);
				glEnd();
			}




			glDisable(GL_BLEND);
		}



		drawLevel();
	}



	/////////////////////////////  2D MODE  //////////////////////////////////////
    
	glActiveTextureARB( GL_TEXTURE1_ARB );
    glDisable(GL_TEXTURE_2D);
	glActiveTextureARB( GL_TEXTURE2_ARB );
    glDisable(GL_TEXTURE_2D);
	glActiveTextureARB( GL_TEXTURE3_ARB );
    glDisable(GL_TEXTURE_2D);
	glActiveTextureARB( GL_TEXTURE0_ARB );
    glEnable(GL_TEXTURE_2D);

	int screenWidth  = CGameEngine::ScreenWidth();
	int screenHeight = CGameEngine::ScreenHeight();

	glMatrixMode( GL_PROJECTION );
	glPushMatrix();
		glLoadIdentity();
		glOrtho(0,screenWidth,0,screenHeight,-1,1);
		glMatrixMode( GL_MODELVIEW );
		glPushMatrix();
			glLoadIdentity();
			//draw
			glDisable(GL_DEPTH_TEST);
			glEnable(GL_BLEND);
			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

			switch(m_eGameState)
			{
			case test:
						if (m_bLeaveQuestion)
						{
							p_yesButton->render();
							p_NoButton->render();

							glBindTexture(GL_TEXTURE_2D, m_iPausingTexture);

							glBegin(GL_QUADS);
								glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f,0.5f);glVertex2f(212.0f,584.0f);
								glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f,1.0f);glVertex2f(212.0f,384.0f);
								glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f,1.0f);glVertex2f(812.0f,384.0f);
								glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f,0.5f);glVertex2f(812.0f,584.0f);
							glEnd();

						}
				break;
				case loadingLevel:
				break;
				case playing:
						if (m_bLeaveQuestion)
						{
							p_yesButton->render();
							p_NoButton->render();
							
							glBindTexture(GL_TEXTURE_2D, m_iPausingTexture);

							glBegin(GL_QUADS);
								glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f,0.5f);glVertex2f(212.0f,584.0f);
								glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f,1.0f);glVertex2f(212.0f,384.0f);
								glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f,1.0f);glVertex2f(812.0f,384.0f);
								glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f,0.5f);glVertex2f(812.0f,584.0f);
							glEnd();
						}
				break;
				case lostGame:
						glBindTexture(GL_TEXTURE_2D, m_iEndingTexture);
						glBegin(GL_QUADS);
							glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f,0.5f);glVertex2f(212.0f,484.0f);
							glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f,1.0f);glVertex2f(212.0f,284.0f);
							glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f,1.0f);glVertex2f(812.0f,284.0f);
							glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f,0.5f);glVertex2f(812.0f,484.0f);
						glEnd();
				break;
				case winGame:
						glBindTexture(GL_TEXTURE_2D, m_iEndingTexture);
						glBegin(GL_QUADS);
							glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f,0.0f);glVertex2f(212.0f,484.0f);
							glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f,0.5f);glVertex2f(212.0f,284.0f);
							glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f,0.5f);glVertex2f(812.0f,284.0f);
							glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f,0.0f);glVertex2f(812.0f,484.0f);
						glEnd();
				break;
			}

			if (m_bLeaveQuestion)
				m_cCursor.render();

			glDisable(GL_BLEND);
			glEnable(GL_DEPTH_TEST);
			// end draw
			glMatrixMode(GL_PROJECTION);
		glPopMatrix();
		glMatrixMode(GL_MODELVIEW);
    glPopMatrix();
}


/****************************************************************************
 *                                                                          *
 *    Function     : update                                                 * 
 *                                                                          *
 *    Description  : Updates the scene                                      *
 *                                                                          *
 *    Interface    : I time = elapsed time since the last automaton cycle   *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CGameProcess::update(Uint32 time)
{
	switch(m_eGameState)
	{
	case loadingLevel:
		m_iTimePast += time;
		if (m_iTimePast > 200 && !m_bReadyToLoad)
			loadLevel();
		if (m_iTimePast > 3000 && m_bLoaded)
		{
			m_eGameState = playing;
			CTextureManager *pTex = CTextureManager::GetInstance();
			pTex->del("level_loading");
		}
		break;

	case playing:
		{
		m_iTimePast += time;
		if (m_iTimePast>400)
		{
			m_iTimePast -=400;
			m_bDrawArrow = !m_bDrawArrow;
		}

		std::map<int,CAnimation*>::const_iterator itr;  // m_vHoleAnimation;
		for(itr = m_vHoleAnimation.begin(); itr != m_vHoleAnimation.end(); ++itr)
		{
				(*itr).second->Update(time);
		}

		
		if (m_iEffectMode == GFX_BAHAMUT)
		{
			m_vFireAnimation[0]->Update(time);
			m_vFireAnimation[1]->Update(time);
			m_vFireAnimation[2]->Update(time);
		}

		switch(m_iBackgroundMode)
		{
			case BKDG_NORMAL:
				break;
			case BKGD_UP_SCROLLING:
				m_fV += time/10000.0f;
				if (m_fV>50.0f) m_fV = 0.0f;
				break;
			case BKGD_LEFT_SCROLLING:
				m_fU += time/10000.0f;
				if (m_fU<-50.0f) m_fU = 0.0f;
				break;
			case BKGD_SPACE:
				m_fV += time/100000.0f;
				m_fU += time/100000.0f;
				if (m_fV>50.0f) m_fV = 0.0f;
				if (m_fU<-50.0f) m_fU = 0.0f;
				break;
		}


		if (!m_bBallInHand && !m_bPaused)
		{
			// Engine simulation step
			m_cPhysic->Update();

			// test if the ball is in a hole
			PObject* obj = m_cPhysic->Object(mi_BallId);
			float x = obj->x;
			float y = obj->y;
			
			if ( y<-50 )
			{
				PObject* obj = m_cPhysic->Object(mi_BallId);
				dBodySetLinearVel(obj->bodyID, 0.0f, 0.0f, 0.0f);
				dBodySetAngularVel(obj->bodyID, 0.0f, 0.0f, 0.0f);
	
				m_cPhysic->SetForce( mi_BallId, 0.0f, 00.0f, 0.0f );
				m_cPhysic->SetTorque( mi_BallId, 0.0f, 00.0f, 0.0f );

				// IN THE HOLE
				int idResult = -1;
				
				std::map<int, CHole *>::const_iterator itr;
				for(itr = m_vHoles.begin(); itr != m_vHoles.end(); ++itr)
				{
					if (x>((*itr).second)->xmin && x <((*itr).second)->xmax)
						idResult = (*itr).first;
				}

				m_iTimePast = 0;
				m_eGameState = waiting;
				CSoundManager *pSound = CSoundManager::GetInstance();

				if (m_iWinningHoleId == idResult)
				{
					//win
					m_bWin = true;
					if (m_iCurrentLvlNb < LEVEL_NUMBER)  // 10 levels
					{
						pSound->PlaySample("win");
						m_iTimePast = 0;
						m_eGameState = waiting;
					}
					else
						m_eGameState = winGame;

					//nextLevel();
				}
				else
				{
					//lost
					m_bWin = false;
					m_iBallNumber--;
					if (m_iBallNumber<0)
					{
						pSound->PlaySample("gameover");
						m_iTimePast = 0;
						m_eGameState = lostGame;
					}
					else
					{
						pSound->PlaySample("lost");
						m_eGameState = waiting;
					}
					
				}
			}

			// Gravitational Holes influence
			if (!m_bBallInHand && !m_bPaused)
			{
				std::map<int, CGravitationalHole *>::const_iterator itr;
				for(itr = m_vGravitationalHoles.begin(); itr != m_vGravitationalHoles.end(); ++itr)
				{
					float xhole = (*itr).second->x;
					float yhole = (*itr).second->y;
					
					PObject* obj = m_cPhysic->Object(mi_BallId);
					float xball = obj->x;
					float yball = obj->y;

					double d = sqrt( (xhole-xball)*(xhole-xball) + (yhole-yball)*(yhole-yball));
					if (d<90)
					{
						float force = 1000.0f/d;
						fprintf(stderr,"attraction ball(%f,%f)  hole(%f,%f) d =%f\n", xball, yball, xhole,yhole, d);
						if (xball < xhole)
                            m_cPhysic->SetGravity(-1.0f,20.5f,0.0f);
						else
							m_cPhysic->SetGravity(+1.0f,20.5f,0.0f);

						/*
						if (xball < xhole)
							dBodyAddForce(obj->bodyID,force, 0.0f,0.0f);
						else
							dBodyAddForce(obj->bodyID,-force, 0.0f,0.0f);*/
					}
					else 
						m_cPhysic->SetGravity(0.0f,-10.0f,0.0f);

				}
			}

		}
		break;
		}
	case waiting:
		m_iTimePast += time;

		if (m_iTimePast > 3000 )
		{
			if (m_bWin)
			{
				CSoundManager *pSound = CSoundManager::GetInstance();
				pSound->PlaySample("nextlevel");
				nextLevel();
			}
			else
			{
				m_eGameState = playing;
				m_bBallInHand = true;

				m_cPhysic->SetPosition(mi_BallId,330.0f,698.0f,0.0f);

				// TODO

			}
		}
	case lostGame:
		m_iTimePast += time;
		if (m_iTimePast > 5000 )
		{
			m_eState = CBaseProcess::Done;
		}

		}
}


/****************************************************************************
 *                                                                          *
 *    Function     : actionNewGame                                          * 
 *                                                                          *
 *    Description  : Button action                                          *
 *                                                                          *
 *    Interface    : I window = number of the window associed to the button *
 *                   I button = button id number                            *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CGameProcess::actionCancel( int window, int button )
{
	m_bPaused = false;
	m_bLeaveQuestion = false;
}

/****************************************************************************
 *                                                                          *
 *    Function     : actionQuit                                             * 
 *                                                                          *
 *    Description  : Button action                                          *
 *                                                                          *
 *    Interface    : I window = number of the window associed to the button *
 *                   I button = button id number                            *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CGameProcess::actionQuit( int window, int button )
{
	m_eState = CBaseProcess::Done;
}

/****************************************************************************
 *                                                                          *
 *    Function     : keyDownEvent                                           * 
 *                                                                          *
 *    Description  : Key down traitment                                     *
 *                                                                          *
 *    Interface    : I key = key down                                       *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CGameProcess::keyDownEvent(SDL_KeyboardEvent* key)
{
	switch( key->keysym.sym ) {
		case SDLK_ESCAPE:
			m_bPaused = true;
			m_bLeaveQuestion = true;
			fprintf(stderr,"key escape\n");
			break;
		case SDLK_SPACE:
			{
				/* DEBUG MODE
				fprintf(stderr, "cheat used\n");
				m_bWin = true;
				m_eGameState = waiting;
				CSoundManager *pSound = CSoundManager::GetInstance();
				pSound->PlaySample("win");
				m_iTimePast = 0;
				*/
			}
		default:
		break;
	}
}


/****************************************************************************
 *                                                                          *
 *    Function     : keyUpEvent                                             * 
 *                                                                          *
 *    Description  : Key up traitment                                       *
 *                                                                          *
 *    Interface    : I key = key up                                         *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CGameProcess::keyUpEvent(SDL_KeyboardEvent* key)
{
}


/****************************************************************************
 *                                                                          *
 *    Function     : mouseButtonDownEvent                                   * 
 *                                                                          *
 *    Description  : Mouse button down traitment                            *
 *                                                                          *
 *    Interface    : I button = button detected                             *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CGameProcess::mouseButtonDownEvent(SDL_MouseButtonEvent* button)
{
	if (m_bLeaveQuestion)
	{
		int screenHeight = CGameEngine::ScreenHeight();
		CPoint2D<int> cursor = CPoint2D<int>(button->x, screenHeight-button->y);
		p_yesButton->hasClick(cursor);
		p_NoButton->hasClick(cursor);
	}

	if ( m_eGameState==playing && m_bBallInHand && !m_bLeaveQuestion)
	{
		m_bBallInHand = false;
		PObject* obj = m_cPhysic->Object( mi_BallId );
		m_cPhysic->SetForce( mi_BallId, 0.0f, -10.0f, 0.0f );
	}
}


/****************************************************************************
 *                                                                          *
 *    Function     : mouseButtonUpEvent                                     * 
 *                                                                          *
 *    Description  : Mouse button up traitment                              *
 *                                                                          *
 *    Interface    : I button = button detected                             *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CGameProcess::mouseButtonUpEvent(SDL_MouseButtonEvent* button)
{
	if (m_bLeaveQuestion)
	{
		int screenHeight = CGameEngine::ScreenHeight();
		CPoint2D<int> cursor = CPoint2D<int>(button->x, screenHeight-button->y);
		p_yesButton->hasUnClick(cursor);
		p_NoButton->hasUnClick(cursor);
	}
}


/****************************************************************************
 *                                                                          *
 *    Function     : mouseMotionEvent                                       * 
 *                                                                          *
 *    Description  : Mouse motion traitment                                 *
 *                                                                          *
 *    Interface    : I motion = motion description                          *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CGameProcess::mouseMotionEvent(SDL_MouseMotionEvent* motion)
{
	if (m_bLeaveQuestion)
	{
		int screenHeight = CGameEngine::ScreenHeight();

		m_cCursor.setPosition(motion->x,screenHeight-motion->y);
		CPoint2D<int> cursor = CPoint2D<int>(motion->x, screenHeight-motion->y);
	}

	if ( m_eGameState==playing && m_bBallInHand)
	{
		PObject* obj = m_cPhysic->Object(mi_BallId);
		float x = obj->x;
		float y = 683;

		x = x + (motion->xrel);
		if ( x<70.0f) x = 70.0f;
		if ( x>730.0f) x = 730.0f;

		obj->x = x;
		obj->y = y;

		m_cPhysic->SetPosition(mi_BallId,x,y,0.0f);


	}
}


/****************************************************************************
 *                                                                          *
 *    Function     : initOGL                                                * 
 *                                                                          *
 *    Description  : Initializes OpenGL area                                *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CGameProcess::initOGL(void)
{
	glClearColor(0.0f,0.0f,0.0f,0.5f);

	glMatrixMode(GL_PROJECTION);											
	glLoadIdentity();														
	gluPerspective(60.0f,1.33f,10.0f,3000.0f);
	glMatrixMode(GL_MODELVIEW);												
	glLoadIdentity();
}


/****************************************************************************
 *                                                                          *
 *    Function     : initGameState                                          * 
 *                                                                          *
 *    Description  : Initializes the game state                             *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CGameProcess::initGameState(void)
{
}

/****************************************************************************
 *                                                                          *
 *    Function     : nextLevel                                              * 
 *                                                                          *
 *    Description  : Changes to the next level                              *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CGameProcess::nextLevel(void)
{
	m_eGameState = loadingLevel;
	m_iTimePast = 0;
	m_iCurrentLvlNb++;

	if (m_iCurrentLvlNb == 11)
		m_eGameState = winGame;
	
	m_bReadyToLoad = false;
	m_bLoaded = false;

	char buffer[50];
	sprintf(buffer, "./levels/level%d/minimap%d.tga",m_iCurrentLvlNb,m_iCurrentLvlNb);

	CTextureManager *pTex = CTextureManager::GetInstance();
	m_iLoadingTexture = pTex->load("level_loading",buffer);
	pTex->del("level_background");
}

/****************************************************************************
 *                                                                          *
 *    Function     : loadLevel                                              * 
 *                                                                          *
 *    Description  : Loads the next level                                   *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CGameProcess::loadLevel(void)
{
	fprintf(stderr, "begin : load level\n");
	m_bReadyToLoad = true;
	m_bBallInHand = true;

	m_iEffectMode = 0;
	m_iBackgroundMode = 0;
	m_iBallNumber = 10;

	m_fU = 0.0f;
	m_fV = 0.0f;

	CSoundManager *pSound = CSoundManager::GetInstance();
	pSound->PauseMusic();

	m_vHoles.clear();
	m_vGravitationalHoles.clear();

	m_cPhysic->Clean(); // Destroy previews objects

	// Left Limit
	int static_object = m_cPhysic->CreateBox(true,100.0f,1500.0f,500.0f);
	m_cPhysic->SetPosition(static_object,0.0f,400.0f,0.0f);
	// Right Limit
	static_object = m_cPhysic->CreateBox(true,100.0f,1500.0f,500.0f);
	m_cPhysic->SetPosition(static_object,800.0f,400.0f,0.0f);

	// front/back Limits
	static_object = m_cPhysic->CreateBox(true,2000.0f,2000.0f,200.0f);
	m_cPhysic->SetPosition(static_object,500.0f,400.0f,-115.0f);
	static_object = m_cPhysic->CreateBox(true,2000.0f,2000.0f,200.0f);
	m_cPhysic->SetPosition(static_object,500.0f,400.0f,+115.0f);


	// THE BALL
	mi_BallId = m_cPhysic->CreateSphere(false,25.0f);
	m_cPhysic->SetPosition(mi_BallId,330.0f,698.0f,0.0f);
	m_cPhysic->SetMass(mi_BallId, 30.0f);

	// XML Parser
	char fileName[50];
	sprintf(fileName, "levels/level%d/level%d.xml",m_iCurrentLvlNb,m_iCurrentLvlNb);
	//sprintf(fileName, "level1.xml");
	TiXmlDocument doc( fileName );
	if ( !doc.LoadFile() )
	{
		// Error: Can't read file.
		fprintf(stderr, "fatal error: XML file %s not found.\n", fileName);
		fprintf(stderr, "error XML : error %d : %s\n", doc.ErrorId(),doc.ErrorDesc());
		return;
	}

	TiXmlHandle hdl(&doc);
	//==================  LEVEL  ==================//
	fprintf(stderr, "load level \n");
	TiXmlElement *XMLelement = hdl.FirstChild( "level").FirstChild( "level").FirstChildElement().Element();
	while (XMLelement)
	{
		std::string strElement	= XMLelement->Value();
		std::string strValue	= XMLelement->GetText();

		if (strElement == "music")
		{
			CSoundManager *pSound = CSoundManager::GetInstance();
			pSound->PlayMusic(strValue.c_str(),true);
			//pSound->PlayMusic("level1",true);
		}

		if (strElement == "ball")
		{
			m_iBallNumber = atoi(strValue.c_str());
			fprintf(stderr,"ball = %d\n", m_iBallNumber);
		}



		if (strElement == "background")
		{
			if (strValue == "normal")		m_iBackgroundMode = BKDG_NORMAL;
			if (strValue == "upscrolling")	m_iBackgroundMode = BKGD_UP_SCROLLING;
			if (strValue == "leftscrolling")m_iBackgroundMode = BKGD_LEFT_SCROLLING;
			if (strValue == "space")		m_iBackgroundMode = BKGD_SPACE;
		}

		if (strElement == "effect")
		{
			if (strValue == "snow")		m_iEffectMode = GFX_SNOW;
			if (strValue == "bubble")	m_iEffectMode = GFX_BUBBLE;
			if (strValue == "plane")	m_iEffectMode = GFX_PLANE;
			if (strValue == "mirage")	m_iEffectMode = GFX_MIRAGE;
			if (strValue == "bahamut")	m_iEffectMode = GFX_BAHAMUT;
		}
		XMLelement = XMLelement->NextSiblingElement();
	}

	//==================  HOLES  ==================//
	fprintf(stderr, "load holes \n");
	int holeId = 0;
	XMLelement = hdl.FirstChild( "level").FirstChild( "holes").FirstChildElement().Element();
	while (XMLelement)
	{
		fprintf(stderr, "new hole \n");
		CHole * hole = new CHole();
		hole->xmin = (float)atof( XMLelement->Attribute("xmin") );
		hole->xmax = (float)atof( XMLelement->Attribute("xmax") );

		m_vHoles[holeId++] = hole;

		XMLelement = XMLelement->NextSiblingElement();
	}

	//================  GRAVI HOLES  ===============//
	fprintf(stderr, "load gravi holes \n");
	int graviholeId = 0;
	XMLelement = hdl.FirstChild( "level").FirstChild( "gravholes").FirstChildElement().Element();
	int index = 0;
	while (XMLelement)
	{
		fprintf(stderr, "new gravi holes \n");
		CGravitationalHole * hole = new CGravitationalHole();
		fprintf(stderr, "get gravi holes \n");
		hole->x = (float)atof( XMLelement->Attribute("x") );
		hole->y = (float)atof( XMLelement->Attribute("y") );
		hole->strengh = (float)atof( XMLelement->Attribute("type") );

		fprintf(stderr, "push gravi holes \n");
		m_vGravitationalHoles[graviholeId++] = hole;

		CAnimation* anin = new CAnimation(hole->x -50.0f, hole->y -50.0f, 50.0f, 100.0f,100.0f, "hole", 24);
		m_vHoleAnimation[index++] = anin;

		XMLelement = XMLelement->NextSiblingElement();
	}

		// TEST
			//int id = m_cPhysic->CreateBox(true, 1000.0f, 50.0f, 100.0f);
			//m_cPhysic->SetPosition(id, 150, 300, 0);
			//m_cPhysic->SetRotation(id, 45, 0, 1, 0);

			//PObject* obj = m_cPhysic->Object(id);
			//dMatrix3  matrix;
			//dRFromAxisAndAngle(matrix, 0, 0, 1, 3.14/4);
			//dGeomSetRotation(obj->geomID, matrix);


	//=============  STATICS OBJECTS  =============//
	fprintf(stderr, "load statics \n");
	XMLelement = XMLelement = hdl.FirstChild( "level").FirstChild( "static").FirstChildElement().Element();
	while (XMLelement)
	{
		int id;
		std::string strType = XMLelement->Attribute("type");
		if (strType == "Box")
		{
			float x = (float)atof( XMLelement->Attribute("x") );
			float y = (float)atof( XMLelement->Attribute("y") );
			float z = (float)atof( XMLelement->Attribute("z") );
			float rx = (float)atof( XMLelement->Attribute("rx") );
			float ry = (float)atof( XMLelement->Attribute("ry") );
			float rz = (float)atof( XMLelement->Attribute("rz") );
			float angle = (float)atof( XMLelement->Attribute("angle") );
			float l = (float)atof( XMLelement->Attribute("l") );
			float L = (float)atof( XMLelement->Attribute("L") );
			float h = (float)atof( XMLelement->Attribute("h") );

			fprintf(stderr, "new box l=%f L=%f h=%f \n",l,L,h);
			id = m_cPhysic->CreateBox(true, l, L, h);
			fprintf(stderr, "box setpos \n");
			m_cPhysic->SetPosition(id, x, y, z);
			fprintf(stderr, "box set Rot \n");
			m_cPhysic->SetRotation(id, angle, x, y, z);
		}

		if (strType == "Sphere")
		{
			float x = (float)atof( XMLelement->Attribute("x") );
			float y = (float)atof( XMLelement->Attribute("y") );
			float z = (float)atof( XMLelement->Attribute("z") );
			float rx = (float)atof( XMLelement->Attribute("rx") );
			float ry = (float)atof( XMLelement->Attribute("ry") );
			float rz = (float)atof( XMLelement->Attribute("rz") );
			float angle = (float)atof( XMLelement->Attribute("angle") );
			float radius = (float)atof( XMLelement->Attribute("radius") );

			fprintf(stderr, "new sphere radius=%f\n", radius);
			id = m_cPhysic->CreateSphere(true, radius);
			m_cPhysic->SetPosition(id, x, y, z);
			m_cPhysic->SetRotation(id, angle, x, y, z);
		}

		if (strType == "Cylinder")
		{
			float x = (float)atof( XMLelement->Attribute("x") );
			float y = (float)atof( XMLelement->Attribute("y") );
			float z = (float)atof( XMLelement->Attribute("z") );
			float rx = (float)atof( XMLelement->Attribute("rx") );
			float ry = (float)atof( XMLelement->Attribute("ry") );
			float rz = (float)atof( XMLelement->Attribute("rz") );
			float angle = (float)atof( XMLelement->Attribute("angle") );
			float radius = (float)atof( XMLelement->Attribute("radius") );
			float length = (float)atof( XMLelement->Attribute("length") );

			fprintf(stderr, "new sphere cylinde rad=%f   len = %f\n", radius, length);
			id = m_cPhysic->CreateCylinder(true, radius,length);
			m_cPhysic->SetPosition(id, x, y, z);
			m_cPhysic->SetRotation(id, angle, x, y, z);
		}

		if (strType == "Plane")
		{
			fprintf(stderr, "new plane\n");
			float a = (float)atof( XMLelement->Attribute("a") );
			float b = (float)atof( XMLelement->Attribute("b") );
			float c = (float)atof( XMLelement->Attribute("c") );
			float d = (float)atof( XMLelement->Attribute("d") );

			//id = m_cPhysic->CreatePlane(a,b,c,d);  //B.U.G
		}

		XMLelement = XMLelement->NextSiblingElement();
	}


	// Calculate the winner Hole
	int n = m_vHoles.size();
	m_iWinningHoleId = rand()%n;
	fprintf(stderr, "n=%d    nb holes = %d      m_iWinningHoleId = %d\n",n,  m_vHoles.size(), m_iWinningHoleId);

	fprintf(stderr, "load textures \n");
	// Unload/Load textures
	CTextureManager *pTex = CTextureManager::GetInstance();
	char buffer[50];
	sprintf(buffer, "./levels/level%d/foreground%d.tga",m_iCurrentLvlNb,m_iCurrentLvlNb);
	pTex->del("level_foreground");
	m_iForegroundTexture = pTex->load("level_foreground",buffer);
	sprintf(buffer, "./levels/level%d/background%d.tga",m_iCurrentLvlNb,m_iCurrentLvlNb);
	pTex->del("level_background");
	m_iBackgroundTexture = pTex->load("level_background",buffer);

	m_bLoaded = true;
	fprintf(stderr, "fin load level \n");
}

/****************************************************************************
 *                                                                          *
 *    Function     : drawLevel                                              * 
 *                                                                          *
 *    Description  : Draws current level                                    *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CGameProcess::drawLevel(void)
{
}


/****************************************************************************
 *                                                                          *
 *    Function     : drawLevel                                              * 
 *                                                                          *
 *    Description  : Draws current level                                    *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CGameProcess::findUVNumber(int number,float &u, float &v)
{
	switch (number)
	{
	case 0:
		u = 0.0f;
		v = 0.0f;
		break;
	case 1:
		u = 0.25f;
		v = 0.0f;
		break;
	case 2:
		u = 0.5f;
		v = 0.0f;
		break;
	case 3:
		u = 0.75f;
		v = 0.0f;
		break;
	case 4:
		u = 0.0f;
		v = 0.25f;
		break;
	case 5:
		u = 0.25f;
		v = 0.25f;
		break;
	case 6:
		u = 0.5f;
		v = 0.25f;
		break;
	case 7:
		u = 0.75f;
		v = 0.25f;
		break;
	case 8:
		u = 0.0f;
		v = 0.5f;
		break;
	case 9:
		u = 0.25f;
		v = 0.5f;
		break;
	}
}

/****************************************************************************
 *                                                                          *
 *  NOTES :                                                                 *
 *                                                                          *
 ****************************************************************************
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *    
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *         
 *                                                                          *    
 ****************************************************************************/ 
 