/****************************************************************************
 *                                                                          *
 *                  Concours 2006 - GAMES CREATORS NETWORK                  *
 *                                                                          *
 ****************************************************************************

                     PRODUCT          : GRAVITY GAME
 
                     FILE             : GameProcess.h

                     VERSION          : 1
 
 ****************************************************************************
 *                                                                          *
 *  DESCRIPTION :                                                           *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          * 
 *                                                                          *  
 ****************************************************************************
 *                                                                          *
 * HISTORY :                                                                * 
 * ---------                                                                *
 *                                                                          *
 *  Date        | Author  | Comment                                         *
 *  ---------------------------------------------------------------------   *
 *  24/07/2006  | Bahamut | Creation                                        *
 *  ---------------------------------------------------------------------   *
 *                                                                          *
 *                                                                          *
 ****************************************************************************/

#ifndef _GAME_PROCESS_H__
#define _GAME_PROCESS_H__ 1

#include "baseprocess.h"
#include <map>
#include "../gui/GPushButton.h"
#include "../gui/GCursor.h"
#include "../physic/PhysicEngine.h"
#include "../3d/Animation.h"

class CHole
{
public:
	float xmin;
	float xmax;
};

class CGravitationalHole
{
public:
	float x;
	float y;
	float strengh;
};

class CGameProcess :
	public CBaseProcess
{
public:
	CGameProcess(void);
	virtual ~CGameProcess(void);
	bool init(void);
	void paint(void);
	void update(Uint32 time);
	void keyDownEvent(SDL_KeyboardEvent* key);
	void keyUpEvent(SDL_KeyboardEvent* key);
	void mouseButtonDownEvent(SDL_MouseButtonEvent* button);
	void mouseButtonUpEvent(SDL_MouseButtonEvent* button);
	void mouseMotionEvent(SDL_MouseMotionEvent* motion);
protected:
	void initOGL(void);
	void initGameState(void);
	void nextLevel();
	void loadLevel();
	void drawLevel();
	void findUVNumber(int number,float &u, float &v);

	// GUI functions (actions)
	void actionCancel( int window, int button ); 
	void actionQuit( int window, int button ); 
	GCursor     m_cCursor;
	GPushButton *p_yesButton;
	GPushButton *p_NoButton;

	//ur physic engine
	CPhysicEngine* m_cPhysic;

	int m_iCurrentLvlNb;
	Uint32 m_iTimePast;
	GLuint m_iLoadingTexture;
	GLuint m_iEndingTexture;
	GLuint m_iPausingTexture;
	GLuint m_iTestTexture;
	GLuint m_iBackgroundTexture;
	GLuint m_iForegroundTexture;
	GLuint m_iBallTexture;
	GLuint m_iHandTexture;
	GLuint m_iholeTexture;
	GLuint m_iNumbersTexture;
	GLuint m_iArrowTexture;

	int m_iBallNumber;
	bool m_bBallInHand;

	bool m_bLeaveQuestion;
	bool m_bReadyToLoad;
	bool m_bLoaded;
	bool m_bPaused;

	int mi_BallId; //temp : id de la balle de test

	std::map<int,CHole*> m_vHoles;
	std::map<int,CGravitationalHole*> m_vGravitationalHoles;
	std::map<int,CAnimation*> m_vHoleAnimation;
	std::map<int,CAnimation*> m_vFireAnimation;
	
	int m_iWinningHoleId;

	int m_iBackgroundMode;
	int m_iEffectMode;

	bool m_bDrawArrow;
	bool m_bWin;

	float m_fU, m_fV;

	enum bkgd  {BKDG_NORMAL=0, BKGD_UP_SCROLLING, BKGD_LEFT_SCROLLING, BKGD_SPACE};
	enum gfx   {GFX_NONE=0,GFX_SNOW, GFX_BUBBLE, GFX_PLANE, GFX_MIRAGE, GFX_BAHAMUT};

	enum state {loadingLevel=0, playing, lostGame, winGame, waiting, test};
	state m_eGameState;


};


#endif



/****************************************************************************
 *                                                                          *
 *  NOTES :                                                                 *
 *                                                                          *
 ****************************************************************************
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *    
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *         
 *                                                                          *    
 ****************************************************************************/