/****************************************************************************
 *                                                                          *
 *                  Concours 2006 - GAMES CREATORS NETWORK                  *
 *                                                                          *
 ****************************************************************************

                     PRODUCT          : GRAVITY GAME
 
                     FILE             : IntroProcess.cpp

                     VERSION          : 1
 
 ****************************************************************************
 *                                                                          *
 *  DESCRIPTION :                                                           *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          * 
 *                                                                          *  
 ****************************************************************************
 *                                                                          *
 * HISTORY :                                                                * 
 * ---------                                                                *
 *                                                                          *
 *  Date        | Author  | Comment                                         *
 *  ---------------------------------------------------------------------   *
 *  24/07/2006  | Bahamut | Creation                                        *
 *  ---------------------------------------------------------------------   *
 *                                                                          *
 *                                                                          *
 ****************************************************************************/

#include "IntroProcess.h"
#include "../managers/SoundManager.h"
#include "../managers/TextureManager.h"



/****************************************************************************
 *                                                                          *
 *    Function     : CIntroProcess                                          * 
 *                                                                          *
 *    Description  : Class construtor                                       *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
CIntroProcess::CIntroProcess(void)
{
	m_eType = CBaseProcess::IntroProcess;
	m_fDelta = 0.0f;
	m_fAlpha = 1.0f; 
}


/****************************************************************************
 *                                                                          *
 *    Function     : ~CCreditProcess                                        * 
 *                                                                          *
 *    Description  : Class Destrutor                                        *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
CIntroProcess::~CIntroProcess(void)
{
}


/****************************************************************************
 *                                                                          *
 *    Function     : init                                                   * 
 *                                                                          *
 *    Description  : Initializes                                            *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value : Returns true if succeed, else returns false            *
 *                                                                          *
 ****************************************************************************/
bool CIntroProcess::init(void)
{
	#if __FLOTTE__
	CTextureManager *pTex = CTextureManager::GetInstance();
	gFlotte = new dmFlotte(pTex->index("intro_water"),pTex->index("intro_logo"), FLOT_HIGHRES);
	#endif

	CSoundManager *pSound = CSoundManager::GetInstance();
	pSound->PlayMusic("intro");
	return true;
}


/****************************************************************************
 *                                                                          *
 *    Function     : paint                                                  * 
 *                                                                          *
 *    Description  : Renders                                                *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CIntroProcess::paint(void)
{
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
	glLoadIdentity();


	gluLookAt(0.0f,0.0f,4.5f,0.1f,0.0f,0.0f,0,1,0);

	glTranslatef(-m_fDelta/3.9f, m_fDelta/3.5f, m_fDelta);
	#if __FLOTTE__
    gFlotte->display();
    #endif

	glDisable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	glColor4f(1.0f,1.0f,1.0f,m_fAlpha);
	glBegin(GL_QUADS);
			glVertex3f(-10.0,-10.0,4.0);
			glVertex3f(+10.0,-10.0,4.0);
			glVertex3f(+10.0,+10.0,4.0);
			glVertex3f(-10.0,+10.0,4.0);
	glEnd();
	glDisable(GL_BLEND);

}


/****************************************************************************
 *                                                                          *
 *    Function     : update                                                 * 
 *                                                                          *
 *    Description  : Updates the scene                                      *
 *                                                                          *
 *    Interface    : I time = elapsed time since the last automaton cycle   *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CIntroProcess::update(Uint32 time)
{
	timeUsed += time;
	if (timeUsed > 8000)
		m_eState = CBaseProcess::Done;

    if (timeUsed > 6000)
	{
		m_fAlpha -= 0.0006f * time;
		if (m_fAlpha<0.0f) m_fAlpha=0.0f;
	}

	m_fDelta += 0.00030f*time;

	#if __FLOTTE__
	gFlotte->runWave(0.0f, 2.0f, 4.0f, 128);
    gFlotte->runWave(400.0f, 5.0f, 3.0f, 64);
    gFlotte->runWave(800.0f, 3.0f, 2.0f, 84);
    gFlotte->runWave(1000.0f, 4.0f, 3.0f, 100);

    gFlotte->update();
	gFlotte->build();
    #endif
}


/****************************************************************************
 *                                                                          *
 *    Function     : keyDownEvent                                           * 
 *                                                                          *
 *    Description  : Key down traitment                                     *
 *                                                                          *
 *    Interface    : I key = key down                                       *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CIntroProcess::keyDownEvent(SDL_KeyboardEvent* key)
{
	switch( key->keysym.sym ) {
		case SDLK_ESCAPE:
			m_eState = CBaseProcess::Done;
			break;
		default:
		break;
	}
}


/****************************************************************************
 *                                                                          *
 *    Function     : keyUpEvent                                             * 
 *                                                                          *
 *    Description  : Key up traitment                                       *
 *                                                                          *
 *    Interface    : I key = key up                                         *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CIntroProcess::keyUpEvent(SDL_KeyboardEvent* key)
{
}


/****************************************************************************
 *                                                                          *
 *    Function     : mouseButtonDownEvent                                   * 
 *                                                                          *
 *    Description  : Mouse button down traitment                            *
 *                                                                          *
 *    Interface    : I button = button detected                             *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CIntroProcess::mouseButtonDownEvent(SDL_MouseButtonEvent* button)
{
}


/****************************************************************************
 *                                                                          *
 *    Function     : mouseButtonUpEvent                                     * 
 *                                                                          *
 *    Description  : Mouse button up traitment                              *
 *                                                                          *
 *    Interface    : I button = button detected                             *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CIntroProcess::mouseButtonUpEvent(SDL_MouseButtonEvent* button)
{
}


/****************************************************************************
 *                                                                          *
 *    Function     : mouseMotionEvent                                       * 
 *                                                                          *
 *    Description  : Mouse motion traitment                                 *
 *                                                                          *
 *    Interface    : I motion = motion description                          *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CIntroProcess::mouseMotionEvent(SDL_MouseMotionEvent* motion)
{
}


/****************************************************************************
 *                                                                          *
 *    Function     : initOGL                                                * 
 *                                                                          *
 *    Description  : Initializes OpenGL area                                *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CIntroProcess::initOGL(void)
{
	glClearColor(1.0f,1.0f,1.0f,1.0f);
	glClearDepth(1.0f);
	glDepthFunc(GL_LEQUAL);
	glEnable(GL_DEPTH_TEST);
	glShadeModel(GL_SMOOTH);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);	

	glMatrixMode(GL_PROJECTION);											
	glLoadIdentity();														
	gluPerspective(60.0f,1.33f,1.0f,3000.0f);
	glMatrixMode(GL_MODELVIEW);												
	glLoadIdentity();

	glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();	
}


/****************************************************************************
 *                                                                          *
 *    Function     : initGameState                                          * 
 *                                                                          *
 *    Description  : Initializes the game state                             *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CIntroProcess::initGameState(void)
{
	timeUsed = 0;
}


/****************************************************************************
 *                                                                          *
 *    Function     : terminate                                              * 
 *                                                                          *
 *    Description  : Terminates the process                                 *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CIntroProcess::terminate(void)
{
	glActiveTextureARB(GL_TEXTURE0_ARB);
	glDisable(GL_TEXTURE_2D);

	glActiveTextureARB(GL_TEXTURE1_ARB);
	glDisable(GL_TEXTURE_2D);

	CSoundManager *pSound = CSoundManager::GetInstance();
	pSound->stopSamples();
	pSound->UnloadMusic("intro");

	CTextureManager *pTex = CTextureManager::GetInstance();
	pTex->del("intro_water");
	pTex->del("intro_logo");
}



/****************************************************************************
 *                                                                          *
 *  NOTES :                                                                 *
 *                                                                          *
 ****************************************************************************
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *    
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *         
 *                                                                          *    
 ****************************************************************************/ 
 
