/****************************************************************************
 *                                                                          *
 *                  Concours 2006 - GAMES CREATORS NETWORK                  *
 *                                                                          *
 ****************************************************************************

                     PRODUCT          : GRAVITY GAME
 
                     FILE             : LoadingProcess.cpp

                     VERSION          : 1
 
 ****************************************************************************
 *                                                                          *
 *  DESCRIPTION :                                                           *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          * 
 *                                                                          *  
 ****************************************************************************
 *                                                                          *
 * HISTORY :                                                                * 
 * ---------                                                                *
 *                                                                          *
 *  Date        | Author  | Comment                                         *
 *  ---------------------------------------------------------------------   *
 *  24/07/2006  | Bahamut | Creation                                        *
 *  ---------------------------------------------------------------------   *
 *                                                                          *
 *                                                                          *
 ****************************************************************************/
 
 
#include "LoadingProcess.h"
#include "../managers/SoundManager.h"
#include "../managers/TextureManager.h"


/****************************************************************************
 *                                                                          *
 *    Function     : CLoadingProcess                                         * 
 *                                                                          *
 *    Description  : Class construtor                                       *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
CLoadingProcess::CLoadingProcess(void)
{
	m_eType = LoadingProcess;
	m_iTimePast = 0;

	CTextureManager *pTex = CTextureManager::GetInstance();
	texture = 0;
}


/****************************************************************************
 *                                                                          *
 *    Function     : ~CLoadingProcess                                        * 
 *                                                                          *
 *    Description  : Class Destrutor                                        *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
CLoadingProcess::~CLoadingProcess(void)
{
}


/****************************************************************************
 *                                                                          *
 *    Function     : init                                                   * 
 *                                                                          *
 *    Description  : Initializes                                            *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value : Returns true if succeed, else returns false            *
 *                                                                          *
 ****************************************************************************/
bool CLoadingProcess::init(void)
{
	CTextureManager *pTex = CTextureManager::GetInstance();
	texture = pTex->load("begin_loading","./textures/loading.jpg");

	return true;
}


/****************************************************************************
 *                                                                          *
 *    Function     : paint                                                  * 
 *                                                                          *
 *    Description  : Renders                                                *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CLoadingProcess::paint(void)
{
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
	glPushMatrix();
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();

		glOrtho(-1.33,1.33,-1.0,1.0,5.0,5.0);
		glMatrixMode(GL_MODELVIEW);

		glEnable(GL_TEXTURE_2D);	

		glActiveTextureARB(GL_TEXTURE1_ARB);
		glDisable(GL_TEXTURE_2D);
    
		glActiveTextureARB(GL_TEXTURE0_ARB);
		glEnable(GL_TEXTURE_2D);

		glBindTexture(GL_TEXTURE_2D, texture); // 2nd texture -> background ..

		// enable texture mapping and specify ourself texcoords
		glDisable(GL_TEXTURE_GEN_S);
		glDisable(GL_TEXTURE_GEN_T);

		glBegin(GL_QUADS);
			glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f,0.75f);glVertex2f(-0.90f,-0.90f);
			glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f,0.75f);glVertex2f(+0.90f,-0.90f);
			glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f,0.0f);glVertex2f(+0.90f,+0.90f);
			glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f,0.0f);glVertex2f(-0.90f,+0.90f);
		glEnd();

	glPopMatrix();
}


/****************************************************************************
 *                                                                          *
 *    Function     : update                                                 * 
 *                                                                          *
 *    Description  : Updates the scene                                      *
 *                                                                          *
 *    Interface    : I time = elapsed time since the last automaton cycle   *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CLoadingProcess::update(Uint32 time)
{
	m_iTimePast += time;
	if (m_iTimePast > 1500)
		m_eState = CBaseProcess::Done;
}


/****************************************************************************
 *                                                                          *
 *    Function     : keyDownEvent                                           * 
 *                                                                          *
 *    Description  : Key down traitment                                     *
 *                                                                          *
 *    Interface    : I key = key down                                       *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CLoadingProcess::keyDownEvent(SDL_KeyboardEvent* key)
{
}


/****************************************************************************
 *                                                                          *
 *    Function     : keyUpEvent                                             * 
 *                                                                          *
 *    Description  : Key up traitment                                       *
 *                                                                          *
 *    Interface    : I key = key up                                         *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CLoadingProcess::keyUpEvent(SDL_KeyboardEvent* key)
{
}


/****************************************************************************
 *                                                                          *
 *    Function     : mouseButtonDownEvent                                   * 
 *                                                                          *
 *    Description  : Mouse button down traitment                            *
 *                                                                          *
 *    Interface    : I button = button detected                             *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CLoadingProcess::mouseButtonDownEvent(SDL_MouseButtonEvent* button)
{
}


/****************************************************************************
 *                                                                          *
 *    Function     : mouseButtonUpEvent                                     * 
 *                                                                          *
 *    Description  : Mouse button up traitment                              *
 *                                                                          *
 *    Interface    : I button = button detected                             *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CLoadingProcess::mouseButtonUpEvent(SDL_MouseButtonEvent* button)
{
}


/****************************************************************************
 *                                                                          *
 *    Function     : mouseMotionEvent                                       * 
 *                                                                          *
 *    Description  : Mouse motion traitment                                 *
 *                                                                          *
 *    Interface    : I motion = motion description                          *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CLoadingProcess::mouseMotionEvent(SDL_MouseMotionEvent* motion)
{
}


/****************************************************************************
 *                                                                          *
 *    Function     : initOGL                                                * 
 *                                                                          *
 *    Description  : Initializes OpenGL area                                *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CLoadingProcess::initOGL(void)
{
	glClearColor(0.0f,0.0f,0.0f,0.5f);
}


/****************************************************************************
 *                                                                          *
 *    Function     : initGameState                                          * 
 *                                                                          *
 *    Description  : Initializes the game state                             *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CLoadingProcess::initGameState(void)
{
}


/****************************************************************************
 *                                                                          *
 *    Function     : terminate                                              * 
 *                                                                          *
 *    Description  : Terminates the process                                 *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CLoadingProcess::terminate(void)
{
	CTextureManager *pTex = CTextureManager::GetInstance();
	pTex->del("begin_loading");
}




/****************************************************************************
 *                                                                          *
 *  NOTES :                                                                 *
 *                                                                          *
 ****************************************************************************
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *    
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *         
 *                                                                          *    
 ****************************************************************************/ 
