/****************************************************************************
 *                                                                          *
 *                  Concours 2006 - GAMES CREATORS NETWORK                  *
 *                                                                          *
 ****************************************************************************

                     PRODUCT          : GRAVITY GAME
 
                     FILE             : MenuProcess.cpp

                     VERSION          : 1
 
 ****************************************************************************
 *                                                                          *
 *  DESCRIPTION :                                                           *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          * 
 *                                                                          *  
 ****************************************************************************
 *                                                                          *
 * HISTORY :                                                                * 
 * ---------                                                                *
 *                                                                          *
 *  Date        | Author  | Comment                                         *
 *  ---------------------------------------------------------------------   *
 *  24/07/2006  | Bahamut | Creation                                        *
 *  ---------------------------------------------------------------------   *
 *                                                                          *
 *                                                                          *
 ****************************************************************************/

#include "MenuProcess.h"
#include "../GameEngine.h"
#include "../managers/SoundManager.h"
#include "../managers/TextureManager.h"

/****************************************************************************
 *                                                                          *
 *    Function     : CMenuProcess                                           * 
 *                                                                          *
 *    Description  : Class construtor                                       *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
CMenuProcess::CMenuProcess(void)
{
	m_eType = CBaseProcess::MenuProcess;
	p_NewGameButton = 0;
	p_QuitButton = 0;
}


/****************************************************************************
 *                                                                          *
 *    Function     : ~CMenuProcess                                          * 
 *                                                                          *
 *    Description  : Class Destrutor                                        *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
CMenuProcess::~CMenuProcess(void)
{
	if (p_NewGameButton) delete p_NewGameButton;
	if (p_QuitButton) delete p_QuitButton;
}


/****************************************************************************
 *                                                                          *
 *    Function     : init                                                   * 
 *                                                                          *
 *    Description  : Initializes                                            *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value : Returns true if succeed, else returns false            *
 *                                                                          *
 ****************************************************************************/
bool CMenuProcess::init(void)
{
	int screenWidth  = CGameEngine::ScreenWidth();
	int screenHeight = CGameEngine::ScreenHeight();

	GCallback<CMenuProcess>* pCallback;

	// Precalculated coordonates for mapping (textures)
	float wSize = 512.0f;       // texture width 
	float hsize = 512.0f;       // texture height
	float ul = 175.0f/wSize;    // long button length
	float vl = 40.0f/hsize;     // long button height
	float us = 100.0f/wSize;    // short button length
	float vs = 40.0f/hsize;     // short button height
	float u0 = 0.0f;            // u coordonates first column
	float u1 = 200.0f/512.0f;   // u coordonates second column
	float u2 = 400.0f/512.0f;   // u coordonates 3th column
	float vi = 50.0f/hsize;     // v coordonates index 


	////////////// BUTTON NEW GAME  //////////////
	p_NewGameButton = new GPushButton(0,screenWidth-270,680,220,50);
	p_NewGameButton->setNormalTexture("menu_buttons",CRectangle<GLfloat>(u0,vi*0,u0+ul,vl+vi*0));
	p_NewGameButton->setPushedTexture("menu_buttons",CRectangle<GLfloat>(u0,vi*1,u0+ul,vl+vi*1));
    pCallback = new GCallback<CMenuProcess>;   // Create Callback
	pCallback->SetClass(this);  
	pCallback->SetAction(actionNewGame);
	p_NewGameButton->initCallback(pCallback);  // and attach Callback to button

	////////////// BUTTON QUIT  //////////////
	p_QuitButton    = new GPushButton(4,screenWidth-270,600,220,50);
	p_QuitButton->setNormalTexture("menu_buttons",CRectangle<GLfloat>(u0,vi*8,u0+ul,vl+vi*8));
	p_QuitButton->setPushedTexture("menu_buttons",CRectangle<GLfloat>(u0,vi*9,u0+ul,vl+vi*9));
	pCallback = new GCallback<CMenuProcess>;
	pCallback->SetClass(this);  
	pCallback->SetAction(actionQuit);
	p_QuitButton->initCallback(pCallback);

	// Cursor
	m_cCursor.load("menu_cursor");
	m_cCursor.setTransparency(1.0);

	// logo GCN
	CTextureManager *pTex = CTextureManager::GetInstance();
	m_iGCNTextureId = pTex->load("minilogo","./textures/minilogoGCN.tga.jpg");



	return true;
}


/****************************************************************************
 *                                                                          *
 *    Function     : paint                                                  * 
 *                                                                          *
 *    Description  : Renders                                                *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CMenuProcess::paint(void)
{

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
	glLoadIdentity();

	gluLookAt(512.0f, 384.0f, 667.0f, 512.0f, 384.0f, 0.0f, 0.0f, 1.0f, 0.0f);
	//gluLookAt(-500.0f,300.0f,40.0f,0.0f,0.0f,0.0f,0,1,0);
	/////////////////////////////  3D MODE  //////////////////////////////////////
	glActiveTextureARB(GL_TEXTURE1_ARB);
	glDisable(GL_TEXTURE_2D);
	glActiveTextureARB(GL_TEXTURE0_ARB);
	glEnable(GL_TEXTURE_2D);


	CTextureManager *pTex = CTextureManager::GetInstance();
	glBindTexture(GL_TEXTURE_2D, pTex->index("boo"));
	glBegin(GL_QUADS);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f,0.0f);glVertex3f(0.0f,768.0f,0.0f);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f,0.72f);glVertex3f(0.0f,0.0f,0.0f);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f,0.72f);glVertex3f(1024.0f,0.0f,0.0f);
				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f,0.0f);glVertex3f(1024.0f,768.0f,0.0f);
	glEnd();


	/////////////////////////////  2D MODE  //////////////////////////////////////
    
	glActiveTextureARB( GL_TEXTURE1_ARB );
    glDisable(GL_TEXTURE_2D);
	glActiveTextureARB( GL_TEXTURE2_ARB );
    glDisable(GL_TEXTURE_2D);
	glActiveTextureARB( GL_TEXTURE3_ARB );
    glDisable(GL_TEXTURE_2D);
	glActiveTextureARB( GL_TEXTURE0_ARB );
    glEnable(GL_TEXTURE_2D);

	int screenWidth  = CGameEngine::ScreenWidth();
	int screenHeight = CGameEngine::ScreenHeight();

	glMatrixMode( GL_PROJECTION );
	glPushMatrix();
		glLoadIdentity();
		glOrtho(0,screenWidth,0,screenHeight,-1,1);
		glMatrixMode( GL_MODELVIEW );
		glPushMatrix();
			glLoadIdentity();
			//draw
			glDisable(GL_DEPTH_TEST);
			glEnable(GL_BLEND);

			// GUI Buttons
			p_NewGameButton->render();
			p_QuitButton->render();


			// Logo GCN
			glBindTexture(GL_TEXTURE_2D, m_iGCNTextureId);
			glBegin( GL_QUADS );
					glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f,1.0f);
					glVertex2f(20.0f,20.0f);
					glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f,1.0f);
					glVertex2f(150.0f,20.0f);
					glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 1.0f,0.0f);
					glVertex2f(150.0f,150.0f);
					glMultiTexCoord2fARB(GL_TEXTURE0_ARB, 0.0f,0.0f);
					glVertex2f(20.0f,150.0f);
			glEnd();

			// Cursor
			m_cCursor.render();
			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

			glDisable(GL_BLEND);
			glEnable(GL_DEPTH_TEST);
			// end draw
			glMatrixMode(GL_PROJECTION);
		glPopMatrix();
		glMatrixMode(GL_MODELVIEW);
    glPopMatrix();
}



/****************************************************************************
 *                                                                          *
 *    Function     : actionNewGame                                          * 
 *                                                                          *
 *    Description  : Button action                                          *
 *                                                                          *
 *    Interface    : I window = number of the window associed to the button *
 *                   I button = button id number                            *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CMenuProcess::actionNewGame( int window, int button )
{
	CSoundManager *pSound = CSoundManager::GetInstance();
	pSound->StopMusic();
	m_eState = CBaseProcess::Done;
}

/****************************************************************************
 *                                                                          *
 *    Function     : actionQuit                                             * 
 *                                                                          *
 *    Description  : Button action                                          *
 *                                                                          *
 *    Interface    : I window = number of the window associed to the button *
 *                   I button = button id number                            *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CMenuProcess::actionQuit( int window, int button )
{
	m_eState = CBaseProcess::Quit;
}


/****************************************************************************
 *                                                                          *
 *    Function     : update                                                 * 
 *                                                                          *
 *    Description  : Updates the scene                                      *
 *                                                                          *
 *    Interface    : I time = elapsed time since the last automaton cycle   *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CMenuProcess::update(Uint32 time)
{
}

/****************************************************************************
 *                                                                          *
 *    Function     : keyDownEvent                                           * 
 *                                                                          *
 *    Description  : Key down traitment                                     *
 *                                                                          *
 *    Interface    : I key = key down                                       *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CMenuProcess::keyDownEvent(SDL_KeyboardEvent* key)
{
	switch( key->keysym.sym ) 
	{
		case SDLK_a:
			break;
		default:
		break;
	}
}


/****************************************************************************
 *                                                                          *
 *    Function     : keyUpEvent                                             * 
 *                                                                          *
 *    Description  : Key up traitment                                       *
 *                                                                          *
 *    Interface    : I key = key up                                         *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CMenuProcess::keyUpEvent(SDL_KeyboardEvent* key)
{
}


/****************************************************************************
 *                                                                          *
 *    Function     : mouseButtonDownEvent                                   * 
 *                                                                          *
 *    Description  : Mouse button down traitment                            *
 *                                                                          *
 *    Interface    : I button = button detected                             *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CMenuProcess::mouseButtonDownEvent(SDL_MouseButtonEvent* button)
{
	int screenHeight = CGameEngine::ScreenHeight();

	CPoint2D<int> cursor = CPoint2D<int>(button->x, screenHeight-button->y);
	p_NewGameButton->hasClick(cursor);
	p_QuitButton->hasClick(cursor);
	
}


/****************************************************************************
 *                                                                          *
 *    Function     : mouseButtonUpEvent                                     * 
 *                                                                          *
 *    Description  : Mouse button up traitment                              *
 *                                                                          *
 *    Interface    : I button = button detected                             *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CMenuProcess::mouseButtonUpEvent(SDL_MouseButtonEvent* button)
{
	int screenHeight = CGameEngine::ScreenHeight();

	CPoint2D<int> cursor = CPoint2D<int>(button->x, screenHeight-button->y);
	p_NewGameButton->hasUnClick(cursor);
	p_QuitButton->hasUnClick(cursor);
}


/****************************************************************************
 *                                                                          *
 *    Function     : mouseMotionEvent                                       * 
 *                                                                          *
 *    Description  : Mouse motion traitment                                 *
 *                                                                          *
 *    Interface    : I motion = motion description                          *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CMenuProcess::mouseMotionEvent(SDL_MouseMotionEvent* motion)
{
	int screenHeight = CGameEngine::ScreenHeight();

	m_cCursor.setPosition(motion->x,screenHeight-motion->y);
	CPoint2D<int> cursor = CPoint2D<int>(motion->x, screenHeight-motion->y);
}


/****************************************************************************
 *                                                                          *
 *    Function     : initOGL                                                * 
 *                                                                          *
 *    Description  : Initializes OpenGL area                                *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CMenuProcess::initOGL(void)
{				
	glClearColor(0.0f,0.0f,0.0f,0.5f);

	glMatrixMode(GL_PROJECTION);											
	glLoadIdentity();														
	gluPerspective(60.0f,1.33f,10.0f,3000.0f);
	glMatrixMode(GL_MODELVIEW);												
	glLoadIdentity();
}


/****************************************************************************
 *                                                                          *
 *    Function     : initGameState                                          * 
 *                                                                          *
 *    Description  : Initializes the game state                             *
 *                                                                          *
 *    Interface    :                                                        *
 *                                                                          *
 *    Return Value :                                                        *
 *                                                                          *
 ****************************************************************************/
void CMenuProcess::initGameState(void)
{
	// Musics
	CSoundManager *pSound = CSoundManager::GetInstance();
	pSound->PlayMusic("menu");
}


/****************************************************************************
 *                                                                          *
 *  NOTES :                                                                 *
 *                                                                          *
 ****************************************************************************
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 ****************************************************************************/ 
 