/****************************************************************************
 *                                                                          *
 *                  Concours 2006 - GAMES CREATORS NETWORK                  *
 *                                                                          *
 ****************************************************************************

                     PRODUCT          : GRAVITY GAME
 
                     FILE             : Point.h

                     VERSION          : 1
 
 ****************************************************************************
 *                                                                          *
 *  DESCRIPTION :                                                           *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          * 
 *                                                                          *  
 ****************************************************************************
 *                                                                          *
 * HISTORY :                                                                * 
 * ---------                                                                *
 *                                                                          *
 *  Date        | Author  | Comment                                         *
 *  ---------------------------------------------------------------------   *
 *  24/07/2006  | Bahamut | Creation                                        *
 *  ---------------------------------------------------------------------   *
 *                                                                          *
 *                                                                          *
 ****************************************************************************/

#ifndef _POINT_H__
#define _POINT_H__


// ==============================================
// CPoint2D - Point 2D
// ==============================================
template<class T> class CPoint2D
{
public:
	inline CPoint2D(): x(0), y(0) {} 
	inline CPoint2D(T X, T Y):x(X), y(Y) {}
	inline CPoint2D(const CPoint2D<T> &v) : x(v.x), y(v.y) { }

	inline T X() const { return x;}
	inline T Y() const { return y;}

	inline void setX(T X) { x = X;}
	inline void setY(T Y) { y = Y;}
	inline void set(T X=0, T Y=0) { x = X; y = Y;}

	inline CPoint2D operator+(const CPoint2D<T> &r) { return CPoint2D<T>(x + r.x,y + r.y);}
	inline CPoint2D operator-(const CPoint2D<T> &r) { return CPoint2D<T>(x - r.x,y - r.y);}

protected:
	T x, y;
};


// ==============================================
// CPoint3D - Point 3D
// ==============================================
template<class T> class CPoint3D
{
public:
	inline CPoint3D(): x(0), y(0), z(0) {} 
	inline CPoint3D(T X, T Y, T Z):x(X), y(Y), z(Z) {}
	inline CPoint3D(const CPoint3D &v) : x(v.x), y(v.y), z(v.z) { }

	inline T X() const { return x;}
	inline T Y() const { return y;}
	inline T Z() const { return z;}

	inline void setX(T X) { x = X;}
	inline void setY(T Y) { y = Y;}
	inline void setZ(T Z) { z = Z;}
	inline void set(T X=0, T Y=0, T Z=0) { x = X; y = Y; z = Z;}

	inline CPoint3D operator+(const CPoint3D<T> &r) { return CPoint3D<T>(x + r.x,y + r.y,z + r.z);}
	inline CPoint3D operator-(const CPoint3D<T> &r) { return CPoint3D<T>(x - r.x,y - r.y,z - r.z);}

protected:
	T x, y, z;
};

#endif


/****************************************************************************
 *                                                                          *
 *  NOTES :                                                                 *
 *                                                                          *
 ****************************************************************************
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *    
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *         
 *                                                                          *    
 ****************************************************************************/