/****************************************************************************
 *                                                                          *
 *                  Concours 2006 - GAMES CREATORS NETWORK                  *
 *                                                                          *
 ****************************************************************************

                     PRODUCT          : GRAVITY GAME
 
                     FILE             : Repere.h

                     VERSION          : 1
 
 ****************************************************************************
 *                                                                          *
 *  DESCRIPTION :                                                           *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          * 
 *                                                                          *  
 ****************************************************************************
 *                                                                          *
 * HISTORY :                                                                * 
 * ---------                                                                *
 *                                                                          *
 *  Date        | Author  | Comment                                         *
 *  ---------------------------------------------------------------------   *
 *  24/07/2006  | Bahamut | Creation                                        *
 *  ---------------------------------------------------------------------   *
 *                                                                          *
 *                                                                          *
 ****************************************************************************/

#ifndef _REPERE_H__
#define _REPERE_H__

#include "Point.h"
#include <math.h>


// ==============================================
// CRepere2D - Repere 2D
// ==============================================
template<class T, class R>
class CRepere2D : virtual public CPoint2D<T> {
public:
	inline CRepere2D(T x=0, T y=0, R phi=0):CPoint2D(x,y)
    {    
      setPhi(phi);
      return ;
    }

	inline R phi(void) const { return _phi; }
	inline R setPhi(R phi)   { return (_phi=phi); }
	inline R phi(R phi)      { return (_phi=phi); }

	inline CRepere2D operator+ (const CRepere2D<T,R> & r)
	{
		return CRepere2D(x()+r.x(), y()+r.y(), phi()+r.phi());
	}

	inline CRepere2D operator- (const CRepere2D<T,R> & r)
	{
		return CRepere2D(x()-r.x(), y()-r.y(), phi()-r.phi());
	}

	inline	CPoint2D<T> operator* (const CPoint2D<T> & r)
	{
		return CPoint2D(x()+r.x()*cos(phi())-r.y()*sin(phi()),y()+r.x()*sin(phi())+r.y()*cos(phi()));
	}

protected:
	R _phi;
};



// ==============================================
// CRepere3D - Repere 3D
// ==============================================
template<class T, class R>
class CRepere3D : public CRepere2D<T, R>, public CPoint3D<T> 
{
public:
  inline CRepere3D(T x=0, T y=0, T z=0, R phi=0, R theta=0, R rho=0):
    CRepere2D(x,y,phi),CPoint3D(x,y,z)
    {    
      setOrientation(phi,theta,rho);
      return ;
    }

  inline R theta(void) const { return _theta; }
  inline R rho(void) const   { return _rho; }
  inline R setTheta(R theta) { return (_theta=theta); }
  inline R setRho(R rho)     { return (_rho=rho); }
  inline R theta(R theta)    { return (_theta=theta); }
  inline R rho(R rho)        { return (_rho=rho); }
  inline void setOrientation(R phi=0, R theta=0, R rho=0) 
  {
	  _phi=phi;
	  _theta=theta;
	  _rho=rho;
	  return;
  }
  
  inline CRepere3D operator+ (const CRepere3D<T,R> & r)
  {
    return CRepere3D(x()+r.x(), y()+r.y(), z()+r.z(),
		  phi()+r.phi(), theta()+r.theta(), rho()+r.rho());
  }

  inline CRepere3D operator- (const CRepere3D<T,R> & r)
  {
    return CRepere3D(x()-r.x(), y()-r.y(), z()-r.z(),
		  phi()-r.phi(), theta()-r.theta(), rho()-r.rho());
  }

protected:
  // TODO: position 3D  dfinir proprement pour chaque objet
  // Proposition : x,y pour le plan (il faut dfinir l'origine)
  //               z: altitude
  //               phi: orientation dans le plan (origine  dfinir)
  //               theta: angle entre le plan et la direction vise
  //                  (positif au dessus du plan)
  //               rho: rotation autour de la direction dfinie prcdemment
  /// angles supplmentaires pour la 3D
  R _theta, _rho;

};

#endif


/****************************************************************************
 *                                                                          *
 *  NOTES :                                                                 *
 *                                                                          *
 ****************************************************************************
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *    
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *         
 *                                                                          *    
 ****************************************************************************/